using System;
using System.Web.UI;


using ScenarioFramework.WorkflowEngine.DataModel.WorkflowProcesses;
using WorkflowServices = ScenarioFramework.WorkflowEngine.DataModel.WorkflowServices;
using WorkflowRelevantData = ScenarioFramework.WorkflowEngine.DataModel.WorkflowRelevantData;
using ACEMPortal.Layer.Workflowlayer.DocumentManager;

using ScenarioFramework.Modules.FormHelper;
using ScenarioFramework.Portal;

namespace SFPortal.Forms
{
	/// <summary>
	/// Descrizione di riepilogo per WaitingPage.
	/// </summary>
	public partial class WaitingPage : System.Web.UI.Page
	{

		private string sAction = string.Empty;
		private string sUser = string.Empty;
		private long lIdProcess = 0;
		private string sIdCCR = string.Empty;
		private string sActivityName = string.Empty;
		private string sProcessName = string.Empty;
		private string sSender = string.Empty;
		private string sResult = string.Empty;
	
		protected void Page_Load(object sender, System.EventArgs e)
		{
			// Inserire qui il codice utente necessario per inizializzare la pagina.
			if ( Page.IsPostBack )
			{
				string sUrl = (string)Session["Redirect"];
				if ( sUrl != null )
				{
					Session["Redirect"] = SFPortal.Redirect.GoHomePage(Page.User.Identity.Name);
					Response.Redirect ( sUrl);
				}
			}
			else
			{
				sAction = Request.QueryString["Action"];
				switch ( sAction )
				{
					case "CreateMainProcess":
						sProcessName = Request.QueryString["ProcessName"];
						break;

					case "CompleteAndWaitingFor":
						sUser = Request.QueryString["User"];
						lIdProcess = Int64.Parse(Request.QueryString["IdProcess"]);
						sActivityName = Request.QueryString["ActivityName"];
						sIdCCR = Request.QueryString["CCR"];
						sSender = Request.QueryString["Sender"];
						sResult = Request.QueryString ["Result"];
						if ( sResult == null)
						{
							sResult = "OK";
						}
						if ( sSender == null )
							sSender = string.Empty;
						break;

					case "ToDo.Messaggi.DO":
					case "ToDo.DoActivity":
					case "ToDo.ActivityRedirect":
						lIdProcess = Int64.Parse(Request.QueryString["IdProcess"]);
						sActivityName = Request.QueryString["ActivityName"];
						break;
				}
			}
		}

		public void MakeAction()
		{
			switch ( sAction )
			{
				case "CreateMainProcess":
					this.CreateMainProcess();
					break;

				case "CompleteAndWaitingFor":
					this.CompleteAndWaitingFor(sResult);
					break;
			}
		}

		private void CreateMainProcess()
		{
			//Creating Process.....
			ScenarioFramework.WAPI.Response response = new ScenarioFramework.WAPI.Response();

			long Proc=0;
            string sAnswer = ScenarioFramework.WAPI.Process.CreateProcessInstance(sProcessName, ref Proc, ref response);
			if (response.iCode !=0)
			{
				Response.Redirect (SFPortal.Redirect.GoError("Process Not Created. [" + response.sMsg + "]"));
			}

            #region Adding Standard Scenario Attributes
            WorkflowRelevantData.XMLAttributes xmlAttributes = new WorkflowRelevantData.XMLAttributes();

            WorkflowRelevantData.XMLAttribute UserAttribute = new WorkflowRelevantData.XMLAttribute("User", "", "");
			UserAttribute.Value = Page.User.Identity.Name;
			xmlAttributes.Add (UserAttribute);

            ScenarioFramework.WAPI.Process.SetProcessInstanceAttributes(xmlAttributes.GetXML(), Proc, ref response);
            #endregion

            if ( response.iCode != 0 )
			{
				Response.Redirect (SFPortal.Redirect.GoError("Process Not Created. [" + response.sMsg + "]"));
			}

			//ScenarioFramework.WAPI.Process.StartProcessInstance(Proc,ref response);
			ScenarioFramework.WAPI.Process.StartProcessInstanceInSynchWay (Proc, ref response);
			if (response.iCode != 0 )
			{
				Response.Redirect (SFPortal.Redirect.GoError("Process Not started[" + response.sMsg + "]"));
			}

			

			XMLActivitiesWithAttributes xmlActivities = new XMLActivitiesWithAttributes(Page.User.Identity.Name);
			
			try
			{
				sAnswer  = ScenarioFramework.WAPI.WorkItem.GetWorkItemList (Page.User.Identity.Name, ref response);
				if (response.iCode==0)
				{
					xmlActivities.Parse(sAnswer);
                    if (xmlActivities.Count == 0)
                    {
                        string sUrlError = SFPortal.Redirect.GoMessage("Il processo  eseguito in modalit synch-way ma non ci sono attivit assegnate all'utente.");
                        Session["Redirect"] = sUrlError;
                    }
                    else
                    {
                        for (int i = 0; i < xmlActivities.Count; i++)
                        {
                            XMLActivity xmlActivity = (XMLActivity)xmlActivities[i];

                            if (xmlActivity.ProcessId == Proc)
                            {
                                string sActionName = "ToDo.ActivityRedirect." + xmlActivity.Name;
                                string ActivityParams = ".aspx?IdProcess=" + xmlActivity.ProcessId + "&IdActivity=" + xmlActivity.Name;
                                //string Path = "../PCMForms/Forms/";
                                string Path = "";

                                FormHelper formHelper = Global.FormsHelper.GetFormHelper();
                                string strAnsware = formHelper.ExecuteAction(Page.User.Identity.Name, sActionName, xmlActivity.ProcessId.ToString(), xmlActivity.Name);
                                if (strAnsware != null && strAnsware.Length > 0 && strAnsware != "OK" && strAnsware != "NO ACTION ALLOWED")
                                {
                                    Session["Redirect"] = Path + strAnsware + ActivityParams;
                                    return;
                                }
                            }
                        }

                        string sUrlError = SFPortal.Redirect.GoMessage("Il processo  eseguito in modalit synch-way ma non ci sono attivit assegnate all'utente.");
                        Session["Redirect"] = sUrlError;
                    }

				}
			}
			catch
			{
				string sUrlError=SFPortal.Redirect.GoError ("Impossible to start process. Check your Messages.");
				Session["Redirect"] = sUrlError;
			}
			
			/*string sActionName = "ToDo.ActivityRedirect." + "FNI";
			string ActivityParams = ".aspx?IdProcess=" + Proc.ToString() + "&IdActivity=" + "FNI";
			string Path = "";

			Scenario.FormHelper.FormHelper formHelper = ACEMPortal.Global.FormsHelper.GetFormHelper();
			string strAnsware = formHelper.ExecuteAction(Page.User.Identity.Name,sActionName,Proc.ToString(),"FNI");
			if (strAnsware!=null && strAnsware.Length>0 && strAnsware!="OK" && strAnsware!="NO ACTION ALLOWED")
			{
				Session["Redirect"] = Path + strAnsware + ActivityParams;
				return;
			}
			*/
		}


		private void CompleteAndWaitingFor(string sResult)
		{
			string sUrlError=string.Empty;
			ScenarioFramework.WAPI.Response response = new ScenarioFramework.WAPI.Response();
			XMLActivity activity = new XMLActivity();
			activity.Name = sActivityName;
			activity.ProcessId = lIdProcess;
			ScenarioFramework.WAPI.WorkItem.CompleteWorkItemSynch(activity.GetXML(), sUser, sResult, ref response);
			if ( response.iCode != 0 )
			{
				Response.Redirect (SFPortal.Redirect.GoError("Impossible to retrieve next step.[" + response.sMsg + "]"));
			}
			
			if ( (sIdCCR == null) || (sIdCCR==string.Empty) )
			{
				//trying to retrieve Process Status.....
				string sProcess = ScenarioFramework.WAPI.Process.GetProcessInstance(ref response, lIdProcess);
				if ( response.iCode == 0 )
				{
					XMLWorkflow workflow = new XMLWorkflow();
					workflow.Parse (sProcess);

					if ( workflow.Status != XMLWorkflow.RUNNING_STATUS )
					{
						string sAttributes = string.Empty;
						sAttributes=ScenarioFramework.WAPI.Process.GetProcessInstanceAttributes(ref response, workflow.ParentProcess);

						if ( response.iCode == 0 ) 
						{
							WorkflowRelevantData.XMLAttributes xmlAttributes = new WorkflowRelevantData.XMLAttributes();
							xmlAttributes.Parse(sAttributes);

							sIdCCR = ((WorkflowRelevantData.XMLAttribute)xmlAttributes.Get("IdCCR")).Value;
						}
					}
				}
			}

			int iEnd =0 ;
			while ( iEnd++ < 30)
			{
				XMLActivitiesWithAttributes xmlActivities = new XMLActivitiesWithAttributes(Page.User.Identity.Name);
					
				try
				{
					string sAnswer  = ScenarioFramework.WAPI.WorkItem.GetWorkItemListRunningNoNotify(Page.User.Identity.Name,ref response);
					if (response.iCode==0)
					{
						xmlActivities.Parse(sAnswer);
						FormHelper formHelper = Global.FormsHelper.GetFormHelper();
						for ( int i=0;i<xmlActivities.Count;i++)
						{
							XMLActivityWithAttributes activityAttr = (XMLActivityWithAttributes)xmlActivities[i];

							if ( (sIdCCR == null) || (sIdCCR==string.Empty) )
							{
								if ( activityAttr.ProcessId == lIdProcess )
								{
									string strAnsware = formHelper.ExecuteAction(sUser,"ToDo.ActivityRedirect.Generic",activityAttr.ProcessId.ToString(),activityAttr.Name);
									string ActivityParams = ".aspx?IdProcess=" + activityAttr.ProcessId + "&IdActivity=" + activityAttr.Name;
									Session["Redirect"] = strAnsware + ActivityParams;
									return;
								}
							}
							else
							{
								if ( activityAttr.IdDocument == sIdCCR )
								{
									if ( sSender.CompareTo("CPR") == 0 )
									{
										//Wrtiting UserRequest Attribute to....
										/*
										BATDocumentManagerHelper DocHelper = new BATDocumentManagerHelper();
										string XmlData = string.Empty;
										try
										{
											//modify by mfarina on 04/10/2005 for DM
											//string sXmlData = DocHelper.GetXMLData(sIdCCR, "CPR");
											string sXmlData = DocHelper.GetXMLData(sIdCCR, "CCR");

											formHelper.LoadXmlAttributes(activityAttr.ProcessId.ToString(),"",Scenario.FormHelper.FormHelper.ACTION_PROCESSATTRIBUTES);

											bool bSaved = false;
											int iPosDot=0;
											WorkflowRelevantData.XMLAttribute attribute = null;
											WorkflowRelevantData.XMLAttributes processAttributes = formHelper.ProcessAttributes;
											for ( int j=0; j<processAttributes.Count; j++)
											{
												attribute = (WorkflowRelevantData.XMLAttribute)processAttributes[j];

												iPosDot = attribute.Name.IndexOf(".");

												if ( iPosDot > -1 )
												{
													string sEntityName = attribute.Name.Substring (0, iPosDot);

													if ( sEntityName.CompareTo ("UserRequest") == 0 )
													{
														string sAttribute = attribute.Name.Substring (iPosDot +1);

														switch ( sAttribute )
														{
															case "Notes":
																attribute.Value = EmebeddedFileHelper.ReadXMLData("txtNotes",sXmlData,false,0,0);
																break;

															case "Date":
																attribute.Value = EmebeddedFileHelper.ReadXMLData("txtDate",sXmlData,false,0,0);
																break;

															case "AgreementDate":
																attribute.Value = EmebeddedFileHelper.ReadXMLData("txtAgreementDate",sXmlData,false,0,0);
																break;

															case "SalesRep":
																attribute.Value = EmebeddedFileHelper.ReadXMLData("ddlSalesRep",sXmlData,false,0,0);
																break;

															case "Customer":
																attribute.Value = EmebeddedFileHelper.ReadXMLData("ddlCustomer",sXmlData,false,0,0);
																break;

															case "PaymentMode":
																attribute.Value = EmebeddedFileHelper.ReadXMLData("ddlPaymentMode",sXmlData,false,0,0);
																break;

															case "TransportMode":
																attribute.Value = EmebeddedFileHelper.ReadXMLData("ddlTransportMode",sXmlData,false,0,0);
																break;

															case "DeliveryMode":
																attribute.Value = EmebeddedFileHelper.ReadXMLData("ddlDeliveryMode",sXmlData,false,0,0);
																break;

															case "PlantType":
																attribute.Value = EmebeddedFileHelper.ReadXMLData("ddlPlantType",sXmlData,false,0,0);
																break;

															case "Stops":
																attribute.Value = EmebeddedFileHelper.ReadXMLData("txtStops",sXmlData,false,0,0);
																break;

															case "HoistwayWidth":
																attribute.Value = EmebeddedFileHelper.ReadXMLData("txtHoistwayWidth",sXmlData,false,0,0);
																break;

															case "HoistwayDepth":
																attribute.Value = EmebeddedFileHelper.ReadXMLData("txtHoistwayDepth",sXmlData,false,0,0);
																break;

															case "Gates":
																attribute.Value = EmebeddedFileHelper.ReadXMLData("txtGates",sXmlData,false,0,0);
																break;

															case "Services":
																attribute.Value = EmebeddedFileHelper.ReadXMLData("txtServices",sXmlData,false,0,0);
																break;
															
															case "Capacity":
																attribute.Value = EmebeddedFileHelper.ReadXMLData("txtCapacity",sXmlData,false,0,0);
																break;

															case "Speed":
																attribute.Value = EmebeddedFileHelper.ReadXMLData("txtSpeed",sXmlData,false,0,0);
																break;

															case "Travel":
																attribute.Value = EmebeddedFileHelper.ReadXMLData("txtTravel",sXmlData,false,0,0);
																break;

															case "Law13":
																attribute.Value =EmebeddedFileHelper.ReadXMLData("chkLaw13",sXmlData,false,0,0);
																break;

															case "PreBuild":
																attribute.Value =EmebeddedFileHelper.ReadXMLData("chkPreBuild",sXmlData,false,0,0);
																break;

															case "HeadRoom":
																attribute.Value = EmebeddedFileHelper.ReadXMLData("txtHeadRoom",sXmlData,false,0,0);
																break;

															case "Pit":
																attribute.Value = EmebeddedFileHelper.ReadXMLData("txtPit",sXmlData,false,0,0);
																break;

															case "Hoistway":
																attribute.Value = EmebeddedFileHelper.ReadXMLData("ddlHoistway",sXmlData,false,0,0);
																break;

															case "PlantNumber":
																attribute.Value =EmebeddedFileHelper.ReadXMLData("txtPlantNumber",sXmlData,false,0,0);
																break;

															case "Figure":
																attribute.Value = EmebeddedFileHelper.ReadXMLData("txtFigure",sXmlData,false,0,0);
																break;

															case "Site":
																attribute.Value = EmebeddedFileHelper.ReadXMLData("txtSite",sXmlData,false,0,0);
																break;

															case "Info":
																attribute.Value = EmebeddedFileHelper.ReadXMLData("txtInfo",sXmlData,false,0,0);
																break;

															case "DriverType":
																attribute.Value =EmebeddedFileHelper.ReadXMLData("ddlDriverType",sXmlData,false,0,0);
																break;

															case "Lawtype":
																attribute.Value = EmebeddedFileHelper.ReadXMLData("ddlLawtype",sXmlData,false,0,0);
																break;

															case "Installer":
																attribute.Value = EmebeddedFileHelper.ReadXMLData("ddlInstaller",sXmlData,false,0,0);
																break;

															case "Peoples":
																attribute.Value = EmebeddedFileHelper.ReadXMLData("txtPeoples",sXmlData,false,0,0);
																break;
														}

														if ( ( attribute.Value == null) || (attribute.Value == string.Empty) )
														{
															switch (BATDB.UserRequest.GetDataType(attribute.Name))
															{
																case BATDBAdapter.BaseDataTypes.TYPE_INTEGER:
																	attribute.Value = "0";
																	break;

																case BATDBAdapter.BaseDataTypes.TYPE_DOUBLE:
																	attribute.Value = "0.0";
																	break;
															}
															
														}

														bSaved = formHelper.SetAttribute(attribute.Name, attribute.Value,Scenario.FormHelper.FormHelper.ACTION_PROCESSATTRIBUTES);
													}
												}
											}
											
											bSaved = formHelper.SaveXmlAttributes (activityAttr.ProcessId.ToString(),"", Scenario.FormHelper.FormHelper.ACTION_PROCESSATTRIBUTES);
											
										}
										catch ( Scenario.Modules.Exception.DocumentManagerException  err) 
										{
											Response.Redirect(ACEMPortal.Redirect.GoError (err.Message));
										}
										*/
									}

									//TODO: 
									string strAnsware= string.Empty;
									switch ( activityAttr.Name ) 
									{
										case "CPR":
											strAnsware = formHelper.ExecuteAction(sUser,"ToDo.ActivityRedirect.CPR",activityAttr.ProcessId.ToString(),activityAttr.Name);
											break;

										case "MPR":
											strAnsware = formHelper.ExecuteAction(sUser,"ToDo.ActivityRedirect.MPR",activityAttr.ProcessId.ToString(),activityAttr.Name);
											break;

										default:
											strAnsware = formHelper.ExecuteAction(sUser,"ToDo.ActivityRedirect.Generic",activityAttr.ProcessId.ToString(),activityAttr.Name);
											break;
									}

									string ActivityParams = ".aspx?IdProcess=" + activityAttr.ProcessId + "&IdActivity=" + activityAttr.Name;
									Session["Redirect"] = strAnsware + ActivityParams;
									return;
								}
							}
						}
					}
				}
				catch
				{
					sUrlError=SFPortal.Redirect.GoError ("Impossible to start process. Check your Messages.");
					Session["Redirect"] = sUrlError;
				}

				//TODO: Inaccettabile
				System.Threading.Thread.Sleep (1000);
			}

			//ActivityNotFound.
			sUrlError=SFPortal.Redirect.GoError ("Impossible to start process. Check your Messages.");
			Session["Redirect"] = sUrlError;
		}

		#region Codice generato da Progettazione Web Form
		override protected void OnInit(EventArgs e)
		{
			//
			// CODEGEN: questa chiamata  richiesta da Progettazione Web Form ASP.NET.
			//
			InitializeComponent();
			base.OnInit(e);
		}
		
		/// <summary>
		/// Metodo necessario per il supporto della finestra di progettazione. Non modificare
		/// il contenuto del metodo con l'editor di codice.
		/// </summary>
		private void InitializeComponent()
		{    

		}
		#endregion

		protected void btnOK_Click(object sender, System.EventArgs e)
		{
			Response.Redirect (SFPortal.Redirect.GoHomePage(Page.User.Identity.Name) );
		}
	}
}
