using System;

using ScenarioFramework.Modules.DM.DataBase;
using ScenarioFramework.Modules.DM.DataModel;

namespace ScenarioFramework.Modules.DM.Application
{
	public class DocumentHelper
	{
		//Possibili tag utilizzabili nell'IdTemplate.
		private const string NAME_TAG			= "Name";
		private const string TYPE_TAG			= "Type";
		private const string PROGRESSIVE_TAG	= "Progressive";
		private const string DATE_TAG			= "Date";
		private const string TIME_TAG			= "Time";
		private const string IDMASTER_TAG		= "IdMaster";
		private const string VERSIONMASTER_TAG	= "VersionMaster";

		private const string YEAR_PROGRESSIVE_TAG    = "YearProgressive";
		private const string MASTER_PROGRESSIVE_TAG  = "MasterProgressive";
		private const string BROTHER_PROGRESSIVE_TAG = "BrotherProgressive";

		// edit by e.leano for DM new relase step 3
		private const string YEAR_TAG ="Year";
		private const string MONTH_TAG= "Month";

		private DocumentManagerDBHelper dmDBHelper;

		public DocumentHelper(DocumentManagerDBHelper dbHelper)
		{
			dmDBHelper = dbHelper;
		}

		public string GetInstanceFileNameWithoutPath(string sDocId, string sVerId, string sFileId, string sFileType)
		{
			return sDocId + "_" + sVerId + "_" + sFileId +"." + sFileType;
		}

		//add step 3
		public string GetInstanceFileName(string sDocId, string sVerId, string sFileId, string sFileType, string sDocName)
		{
			XMLDocumentTemplate xmlTemplate= this.dmDBHelper.GetDocumentTemplate(sDocName);
			string sPath = dmDBHelper.GetInstanceDirectory();
			sPath+= this.ResolveSavingMode(xmlTemplate,"");


			sPath+= "Files\\";
			sPath += this.GetInstanceFileNameWithoutPath(sDocId,sVerId,sFileId, sFileType);

			return DocumentHelper.EraseTwiceSlashInPath(sPath);
		}
		public string GetInstanceFileName(string sDocId, string sVerId, string sFileId, string sFileType)
		{
			string sPath = dmDBHelper.GetInstanceDirectory();
			

			XMLAttributesHelper xmlAttributes = dmDBHelper.LoadDocumentAttributes(sDocId, sVerId );
			string sStorage = string.Empty;
			if ( xmlAttributes != null)
			{
				try
				{
					ScenarioFramework.WorkflowEngine.DataModel.WorkflowRelevantData.XMLAttribute xmlAttribute = xmlAttributes.Get ( "DocumentStorage");
					if ( xmlAttribute != null) 
					{
						sStorage = xmlAttribute.Value;
					}
				}
				catch ( ScenarioFramework.WorkflowEngine.DataModel.Exception.AttributeNotFound )
				{
				}
			}
			sPath+=sStorage;
			sPath+= "Files\\";
			sPath += this.GetInstanceFileNameWithoutPath(sDocId,sVerId,sFileId, sFileType);
			return DocumentHelper.EraseTwiceSlashInPath(sPath);
		}

		
		public string GetInstanceFileName(string sDocId)
		{
			XMLDocumentInstance document = dmDBHelper.GetDocumentInstance (sDocId);

			return GetInstanceFileName(sDocId, document.XMLDocumentVersions.Count.ToString(), document.XMLDocumentFile.Id, document.XMLDocumentFile.Type, document.Name);
		}

		public string GetInstanceFileNameWithoutPath(string sDocId)
		{
			XMLDocumentInstance document = dmDBHelper.GetDocumentInstance (sDocId);

			return GetInstanceFileNameWithoutPath(sDocId, document.XMLDocumentVersions.Count.ToString(), document.XMLDocumentFile.Id, document.XMLDocumentFile.Type);
		}
		
		public string GetTemplateFileName(string sDocName, string sFileId, string sFileType)
		{
			string sPath = dmDBHelper.GetTemplateFilePath();

			//sPath += sDocName + "_" + sFileId;
			sPath += sFileId + "." + sFileType;

			return DocumentHelper.EraseTwiceSlashInPath(sPath);
		}

		public string ResolveId(string sIdTemplate, XMLDocumentInstance docInstance, out bool recycled)
		{
			string sId=string.Empty;
			string sTagName;
			int iPosTagB;
			int iPosTagE=0;
			int iPosCur=0;
			recycled=false;
			while ( (iPosTagB=sIdTemplate.IndexOf("[", iPosCur )) != -1 )
			{
				sId+=sIdTemplate.Substring( iPosCur, iPosTagB - iPosCur);

				if ( ( iPosTagE = sIdTemplate.IndexOf("]", iPosTagB + 1 ) ) != -1 )
				{
					sTagName = sIdTemplate.Substring(iPosTagB + 1, ( iPosTagE - iPosTagB - 1) );
					
					long lDocumentProgressive =0;
					switch ( sTagName )
					{
						case DocumentHelper.NAME_TAG:
							sId+=docInstance.Name;
							break;

						case DocumentHelper.TYPE_TAG:
							sId+=docInstance.Type;
							break;

						case DocumentHelper.YEAR_TAG:
							sId+=DateTime.Now.Year.ToString();
							break;

						
						/*case DocumentHelper.PROGRESSIVE_TAG:
							sId+=DocumentManagerDBHelper.GetDocumentProgressive().ToString();
							break;
							*/

						case DocumentHelper.PROGRESSIVE_TAG:
							lDocumentProgressive = dmDBHelper.GetDocumentProgressiveFromDB(docInstance.Name, out recycled);
							sId+=String.Format ("{0:000}",lDocumentProgressive);
							break;
						

						case DocumentHelper.DATE_TAG:
							sId+=DateTime.Now.ToShortDateString();
							break;

						case DocumentHelper.TIME_TAG:
							sId+=DateTime.Now.ToShortTimeString();
							break;

						case DocumentHelper.IDMASTER_TAG:
							sId+=docInstance.IdMaster;
							break;

						case DocumentHelper.VERSIONMASTER_TAG:
							sId+=docInstance.VersionMaster;
							break;

						case DocumentHelper.YEAR_PROGRESSIVE_TAG:
							lDocumentProgressive = dmDBHelper.GetDocumentYearProgressive(docInstance.Name);
							sId+=String.Format ("{0:000}",lDocumentProgressive);
							break;
						
						case DocumentHelper.BROTHER_PROGRESSIVE_TAG:
							lDocumentProgressive = dmDBHelper.GetDocumentProgressiveFromDB(docInstance.Name+docInstance.IdMaster, out recycled);
							sId+=String.Format ("{0:000}",lDocumentProgressive);
							break;
						
						case DocumentHelper.MASTER_PROGRESSIVE_TAG:
							lDocumentProgressive = dmDBHelper.GetDocumentProgressiveFromDB(docInstance.IdMaster, out recycled);
							sId+=String.Format ("{0:000}",lDocumentProgressive);
							break;
						default:
							int iPosProgressive = -1;
							iPosProgressive = sTagName.IndexOf (DocumentHelper.PROGRESSIVE_TAG);
							string sProgressiveDocument = string.Empty;

							if ( iPosProgressive >-1) 
							{
								sProgressiveDocument = sTagName.Substring (iPosProgressive + DocumentHelper.PROGRESSIVE_TAG.Length +1);
								lDocumentProgressive = DocumentManagerDBHelper.GetDocumentProgressive(sProgressiveDocument);
								sId+=String.Format ("{0:000}",lDocumentProgressive);
							}
							break;

							
					}
		            
					iPosCur = iPosTagE+1;

					if ( iPosCur == sIdTemplate.Length ) 
						break;
				}
				else
				{
					iPosCur=++iPosTagB;
				}
			} 

			if ( iPosCur <= sIdTemplate.Length ) 
				sId+=sIdTemplate.Substring(iPosCur);

			return sId;

		}

		//modified by e.leano for new version of DM: step 3
	
		public string ResolveSavingMode(XMLDocumentTemplate docTemplate, string sIdMaster)
		{
			string sId=string.Empty;
			string sTagName;
			int iPosTagB;
			int iPosTagE=0;
			int iPosCur=0;
			int year=0;
			int month=0;
			int document=0;
			string sSaveMode=string.Empty;

			while ( (iPosTagB=docTemplate.DocumentStorage.IndexOf("[", iPosCur )) != -1 )
			{
				sId+=docTemplate.DocumentStorage.Substring( iPosCur, iPosTagB - iPosCur);

				if ( ( iPosTagE = docTemplate.DocumentStorage.IndexOf("]", iPosTagB + 1 ) ) != -1 )
				{
					sTagName = docTemplate.DocumentStorage.Substring(iPosTagB + 1, ( iPosTagE - iPosTagB - 1) );
					
					switch ( sTagName )
					{
						case DocumentHelper.YEAR_TAG:
							year=1;
							break;

						case DocumentHelper.MONTH_TAG:
							month=1;
							break;

						
						case DocumentHelper.IDMASTER_TAG:
							document=1;
							break;
						
												
					}
		            
					iPosCur = iPosTagE+1;

					if ( iPosCur == docTemplate.DocumentStorage.Length ) 
						break;
				}
				else
				{
					iPosCur=++iPosTagB;
				}
			} 
			if(year==1)
			{
				sSaveMode+=System.DateTime.Now.Year.ToString();
				sSaveMode+="\\";
			}
			if (month==1)
			{
				sSaveMode+=this.MonthName(System.DateTime.Now.Month);
				sSaveMode+="\\";
			}
			if (document==1)
			{
				sSaveMode+=sIdMaster;
				sSaveMode+="\\";
			}
		
			return sSaveMode;

		}
		//modified by e.leano for new version of DM: step 3
		private string MonthName(int month) 
		{
			switch (month) 
			{
				case  1: return ("Gennaio");
				case  2: return ("Febbraio");
				case  3: return ("Marzo");
				case  4: return ("Aprile");
				case  5: return ("Maggio");
				case  6: return ("Giugno");
				case  7: return ("Luglio");
				case  8: return ("Agosto");
				case  9: return ("Settembre");
				case 10: return ("Ottobre");
				case 11: return ("Novembre");
				case 12: return ("Dicembre");
				
				
			}
			return string.Empty;
		}

		public XMLDocumentVersion AddVersion(XMLDocumentInstance document)
		{
			string sFileOldVersion = string.Empty;
			string sFileNewVersion = string.Empty;

			XMLDocumentVersion versionAdded =document.XMLDocumentVersions.AddVersion ();
			switch ( document.CreationPolicy)
			{
				case XMLDocumentBase.NEW_CREATIONPOLICY:
					sFileOldVersion = this.GetTemplateFileName(document.Name, document.XMLDocumentFile.Id, document.XMLDocumentFile.Type);
					sFileNewVersion = this.GetInstanceFileName(document.Id, versionAdded.Id, document.XMLDocumentFile.Id, document.XMLDocumentFile.Type,document.Name);

					//Make directories if not exists
					this.dmDBHelper.GetDocPath (document);
					System.IO.File.Copy(sFileOldVersion, sFileNewVersion, true);	//add true
					break;

				case XMLDocumentBase.MIGRATE_CREATIONPOLICY:
					break;

				case XMLDocumentBase.PREVOIUS_CREATIONPOLICY:
					XMLDocumentVersion lastVersion = document.XMLDocumentVersions.GetLastVersion();
					if ( lastVersion != null )
					{
						versionAdded.XMLDocuments.Parse ( lastVersion.XMLDocuments.GetXML());
						
						sFileOldVersion = this.GetInstanceFileName(document.Id, lastVersion.Id, document.XMLDocumentFile.Id, document.XMLDocumentFile.Type);
						sFileNewVersion = this.GetInstanceFileName(document.Id, versionAdded.Id, document.XMLDocumentFile.Id, document.XMLDocumentFile.Type);

						System.IO.File.Copy(sFileOldVersion, sFileNewVersion);

						XMLDocumentFile docFile = null;
						for ( int i=0; i<lastVersion.XMLDocumentAttachFiles.Count;i++)
						{
							docFile = (XMLDocumentFile)lastVersion.XMLDocumentAttachFiles[i];

							if ( docFile.MigratePolicy == XMLDocumentFile.VERSION_MIGRATE_POLICY )
							{
								XMLDocumentFile newFile = new XMLDocumentFile();
								newFile.Parse ( docFile.GetXML() );

								versionAdded.XMLDocumentAttachFiles.Add ( newFile );

								sFileOldVersion = this.GetInstanceFileName(document.Id, lastVersion.Id, docFile.Id, docFile.Type );
								sFileNewVersion = this.GetInstanceFileName(document.Id, versionAdded.Id, newFile.Id, newFile.Type);

								System.IO.File.Copy(sFileOldVersion, sFileNewVersion);

							}
						}
					}

					break;
			}

			return versionAdded;
		}

		
		public void MigrateEmbedded( string sFileInput,string sFileOutput, string sFileTransfer)
		{
			/*
			XmlDocument MyFromModule = new XmlDocument();
			XmlDocument MyToModule = new XmlDocument();
			XmlDocument MyTransfer = new XmlDocument();

			try
			{
				MyFromModule.LoadXml(sFileInput);
				MyToModule.LoadXml(sFileOutput);
				MyTransfer.LoadXml(sFileTransfer);

				XmlNode rootTransfer=MyTransfer.FirstChild;

				for (int i=0; i< rootTransfer.ChildNodes.Count;++i)
				{
					XmlNode root;
					root=rootTransfer.ChildNodes[i];

					// Parte la migrazione

									
					// cerca il nodo ITEM NUMBER nel modulo to

					XmlNode FromItemNumber;
					FromItemNumber = MyFromModule.SelectSingleNode("Module/IsItemNumber");
									
					XmlNode ToItemNumber;
					ToItemNumber = MyToModule.SelectSingleNode("Module/IsItemNumber");
										

					if ((root.Attributes["FromItemNumber"].Value==FromItemNumber.InnerText) && (root.Attributes["ToItemNumber"].Value==ToItemNumber.InnerText))
					{

						// selsziona il tipo di nodo e la modifica da effetture
						if (root.Attributes["FromIsMatrixField"].Value=="YES")
						{
							if (root.Attributes["ToIsMatrixField"].Value=="YES")
							{
								// controlla se il trasferimento  di tutta la matrice
								if (root.Attributes["AllMatrix"].Value=="YES")
								{
									//caso di trasferimento di tutta la matrice

									// cerca il nodo nel file di from
									string FieldFromName = root.Attributes["FromMatrixName"].Value;
									XmlNode FieldFrom;
									FieldFrom = MyFromModule.SelectSingleNode("Module/Matrixes/" + FieldFromName);

									// cerca il nodo nel file di to
									string FieldToName = root.Attributes["ToMatrixName"].Value ;
									XmlNode FieldTo;
									FieldTo = MyToModule.SelectSingleNode("Module/Matrixes/" + FieldToName);

									//effettua il transfer

									FieldTo.InnerXml = FieldFrom.InnerXml;			
								}
								else
								{
									// caso in cui il from  un item matrice ed il to  un item matrice

									// cerca il nodo nel file di from
									string FieldFromName = root.Attributes["FromMatrixName"].Value + "/Row" + root.Attributes["FromRow"].Value + "/Col" + root.Attributes["FromCol"].Value;
									XmlNode FieldFrom;
									FieldFrom = MyFromModule.SelectSingleNode("Module/Matrixes/" + FieldFromName);

									// cerca il nodo nel file di to
									string FieldToName = root.Attributes["ToMatrixName"].Value + "/Row" + root.Attributes["ToRow"].Value + "/Col" + root.Attributes["ToCol"].Value;
									XmlNode FieldTo;
									FieldTo = MyToModule.SelectSingleNode("Module/Matrixes/" + FieldToName);
												
									//effettua il transfer
									FieldTo.InnerText = FieldFrom.InnerText;
								}
							}
							else
							{
								// caso in cui il from  un item matrice ed il to  un campo

								// cerca il nodo nel file di from
								string FieldFromName = root.Attributes["FromMatrixName"].Value + "/Row" + root.Attributes["FromRow"].Value + "/Col" + root.Attributes["FromCol"].Value;
								XmlNode FieldFrom;
								FieldFrom = MyFromModule.SelectSingleNode("Module/Matrixes/" + FieldFromName);

								// cerca il nodo nel file di to
								string FieldToName = root.Attributes["ToFieldName"].Value;
								XmlNode FieldTo;
								FieldTo = MyToModule.SelectSingleNode("Module/Fields/" + FieldToName);

								//effettua il transfer
								FieldTo.InnerText = FieldFrom.InnerText;
							}
						}
						else
						{
							if (root.Attributes["ToIsMatrixField"].Value=="YES")
							{
								// caso in cui il from  un campo ed il to  un item matrice
												
								// cerca il nodo nel file di from
								string FieldFromName = root.Attributes["FromFieldName"].Value;
								XmlNode FieldFrom;
								FieldFrom = MyFromModule.SelectSingleNode("Module/Fields/" + FieldFromName);
										
								// cerca il nodo nel file di to
								string FieldToName = root.Attributes["ToMatrixName"].Value + "/Row" + root.Attributes["ToRow"].Value + "/Col" + root.Attributes["ToCol"].Value;
								XmlNode FieldTo;
								FieldTo = MyToModule.SelectSingleNode("Module/Matrixes/" + FieldToName);

								//effettua il transfer
								FieldTo.InnerText = FieldFrom.InnerText;
							}
							else
							{
								// caso in cui il from  un campo ed il to  un campo

								// cerca il nodo nel file di from
								string FieldFromName = root.Attributes["FromFieldName"].Value;
								XmlNode FieldFrom;
								FieldFrom = MyFromModule.SelectSingleNode("Module/Fields/" + FieldFromName);
										
								// cerca il nodo nel file di to
								string FieldToName = root.Attributes["ToFieldName"].Value;
								XmlNode FieldTo;
								FieldTo = MyToModule.SelectSingleNode("Module/Fields/" + FieldToName);

								//effettua il transfer
								FieldTo.InnerText = FieldFrom.InnerText;
							}
						}
					}
								
				}

				// salva il file
				MyToModule.Save(DestinationName);
				SourceReader.Close();
			}
			catch
			{
				// Risponde con il messaggio di errore
				MyFromModule = null;
				SourceReader.Close();
			}
			*/
		}

		
		public static int Copy(string sFileSource, string sFileTarget)
		{
			if ( System.IO.File.Exists ( sFileTarget ) )
			{
				System.IO.File.Delete ( sFileTarget );
			}

			System.IO.File.Copy (sFileSource, sFileTarget);

			return 0;
		}

		
		private static string EraseTwiceSlashInPath(string sPath)
		{
			string sPathResult = string.Empty;
			bool bSlash=false;
			for ( int i=0; i<sPath.Length;i++)
			{
				if ( sPath[i] == '\\') 
				{
					if ( bSlash == false )
					{
						bSlash=true;
						sPathResult+=sPath[i];
					}
				}
				else
				{
					bSlash = false;
					sPathResult+=sPath[i];
				}
			}

			return sPathResult;
		}




	}
}
