using System;
using System.Xml;
using System.Xml.XPath;
using System.Xml.Xsl;
using ScenarioFramework.WorkflowEngine.DataModel.WorkflowProcesses;
using ScenarioFramework.WorkflowEngine.DataModel.WorkflowRelevantData;
using ScenarioFramework.Modules.PCM.DataModel.ConfigurationManager.Entity;
using ScenarioFramework.Modules.PCM.DataModel.ConfigurationManager.Property;

using System.IO;
using System.Text;
using ScenarioFramework.WorkflowEngine.DataModel.WorkflowParticipantsData;
using ScenarioFramework.Modules.DM.DataBase;
using ScenarioFramework.Modules.DM.Application;
using ScenarioFramework.WorkflowEngine.DataModel.WorkflowDataTypes;
//using FormAttributeActions;

namespace ScenarioFramework.Modules.FormHelper
{
	//**************************************************************************************
	// 
	// FormHelper Class
	//
	// The FormHelper class rappresents a method-set to offer help functions to forms from
	// Web applications that calls it to obtain execution of commands on processes or activities.
	// Forms can also obtains redirect url to switch their page cntrol after command execution.
	//
	//**************************************************************************************
	public class FormHelper
	{
		//only ITPortal
		private XMLActions allowedActions;

		private XMLAttributes FH_ActivityAttributes;
		private XMLAttributes FH_ProcessAttributes;
		
		// PCM Attributes will contain configuration values like xpath "EntityNme.AttributeName"
		// to rappresent an attribute value or "EntityName" to rappresente an entity value
		// got from ITEMS database
		private XMLEntities FH_PCM_Attributes;

		// PCM Entities will contain form activity graphic configuration like Visible Enabled
		// DataInsert or DataObtain to help form to autoconfigure its visualization mode
		private XMLEntities FH_PCM_Entities;

		// General
		public const string ACTION_HOME = "HomePage";

		// ConfigurationManager
		public const string ACTION_PCMATTRIBUTES			="PCM_Attributes";
		public const string ACTION_PCMENTITIES				="PCM_Entities";

		// Process Actions
		public const string ACTION_PROCESSATTRIBUTES		= "ProcessAttributes";
		public const string ACTION_PROCESSGET				= "ProcessGet";
		public const string ACTION_PROCESSDO				= "ProcessDo";

		// Activity Actions
		public const string ACTION_ACTIVITYATTRIBUTES		= "ActivityAttributes";
		public const string ACTION_ACTIVITYGET				= "ActivityGet";
		public const string ACTION_ACTIVITYDO				= "ActivityDo";

		// WorkItem Actions
		public const string ACTION_WORKITEMGET				= "WorkItemGet";
		public const string ACTION_WORKITEMDO				= "WorkItemDo";

		public FormHelper(XMLActions actions)
		{
			allowedActions = actions;
		}

		public XMLActions XMLActions
		{
			get { return allowedActions;}
			set { allowedActions = value;}
		}

		public XMLEntities PCM_Entities
		{
			get {return FH_PCM_Entities;}
		}
		
		public XMLAttributes ProcessAttributes
		{
			get { return FH_ProcessAttributes;}
		}

		public XMLEntities PCM_Attributes
		{
			get { return FH_PCM_Attributes;}
		}

		public string GetDocumentPage(string sMenuPath, string sUser)
		{
			//Found User HomePage
			string sAction =sMenuPath + ".Menu";
			XMLAction action = null;
			try
			{
				action = allowedActions.Get(sAction);
				return action.RedirectUrl;
					
			}
			catch ( Exception.XMLActionNotFound )
			{}

			return string.Empty;
		}

		public string GetHomePage(string sUser)
		{
			//Found User HomePage
			string sAction =sUser + ".HomePage";
			XMLAction action = null;
			try
			{
				action = allowedActions.Get(sAction);
				return action.RedirectUrl;
					
			}
			catch ( Exception.XMLActionNotFound )
			{}

			//trying to search for Role.
			WAPI.Response wapiResponse = new WAPI.Response();
			string sRoles = WAPI.Role.GetRolesByUser(sUser, ref wapiResponse);
			if ( wapiResponse.iCode == 0 ) 
			{
				XMLRoles roles = new XMLRoles();
				roles.Parse(sRoles);

				for ( int i=0; i<roles.Count;i++)
				{
					XMLRole role = (XMLRole)roles[i];
					sAction = role.Key + ".HomePage";

					try
					{
						action = allowedActions.Get(sAction);

						if ( action != null )
							return action.RedirectUrl;
					}
					catch(Exception.XMLActionNotFound)
					{}
				}

			}
			
			return string.Empty;
		}

		
		// ################################################################################################
		// ##
		// ## LoadXmlAttributes loads selected attributes type from activity or process or pcm attributes
		// ## and puts them into formhelper module variables FH_ActivityAttributes, FH_ProcessAttributes
		// ## FH_PCM_Entities FH_PCM_Attributes.
		// ##
		// ## Usually a FormHelper Istance has to execute this sequense before any other command to congiure variables
		// ## LoadXmlAttributes("ID","","ProcessAttributes");  To load Process attributes into  Variable
		// ## LoadXmlAttributes("ID","","ProductConfigurationManagerAttributes");  To load Process attributes into  Variable
		// ## LoadXmlAttributes("ID","Name","ActivityAttributes");  To load Process attributes into  Variable
		// ## LoadXmlAttributes("ID","Name","ProductConfigurationManagerEntities");  To load Process attributes into  Variable
		// ##
		// ################################################################################################
		public void LoadXmlAttributes(string ProcessId, string ActivityName, string AttributeActionType)
		{
			// Vallidate input
			if (AttributeActionType==null || AttributeActionType.Length==0)
			{
				throw (new ArgumentOutOfRangeException("FormHelper - LoadXmlAttributes - No AttributeActionType"));
			}

			// select witch type of load action on attributes to do
			switch (AttributeActionType)
			{
				case ACTION_PROCESSATTRIBUTES:
					// Vallidate input
					if (ProcessId==null || ProcessId.Length==0)
					{
						throw (new ArgumentOutOfRangeException("FormHelper - LoadXmlAttributes - No ProcessId"));
					}

					// load process attributes in module variable
					try
					{
						FH_ProcessAttributes = new XMLAttributes();
						FH_ProcessAttributes.Parse(this.GetProcessAttributes(ProcessId));
					}
					catch
					{
						FH_ProcessAttributes = null;
					}
					break;

				case ACTION_ACTIVITYATTRIBUTES:
					// Vallidate input
					if (ProcessId==null || ProcessId.Length==0)
					{
						throw (new ArgumentOutOfRangeException("FormHelper - LoadXmlAttributes - No ProcessId"));
					}
					
					if (ActivityName==null || ActivityName.Length==0)
					{
						throw (new ArgumentOutOfRangeException("FormHelper - LoadXmlAttributes - No ActivityName"));
					}

					// load activity attributes in module variable
					try
					{
						FH_ActivityAttributes = new XMLAttributes();
						FH_ActivityAttributes.Parse(this.GetActivityAttributes(ProcessId,ActivityName));
					}
					catch
					{
						FH_ProcessAttributes = null;
					}
					break;

				case ACTION_PCMENTITIES:
					// Vallidate input
					if (FH_ActivityAttributes==null || FH_ActivityAttributes.Count==0)
						throw (new ArgumentOutOfRangeException("FormHelper - LoadXmlAttributes - No PCM Entities"));
					
					// load PCM attributes in module variable
					try
					{
						FH_PCM_Entities = new XMLEntities ();
						FH_PCM_Entities.Parse(this.GetAttribute("PCM_Entities",ACTION_PCMENTITIES));
					}
					catch
					{
						FH_PCM_Entities = null;
					}
					
					;
					break;

				case ACTION_PCMATTRIBUTES:
					// Vallidate input
					if (FH_ProcessAttributes==null || FH_ProcessAttributes.Count==0)
					{
						throw (new ArgumentOutOfRangeException("FormHelper - LoadXmlAttributes - No PCM Attributes"));
					}
					
					// load PCM attributes in module variable
					try
					{
						FH_PCM_Attributes = new XMLEntities ();
						FH_PCM_Attributes.Parse(this.GetAttribute("PCM_Attributes",ACTION_PCMATTRIBUTES));
					}
					catch
					{
						FH_PCM_Attributes = null;
					}
					
					break;

			}
		}

		public void LoadXmlAttributes(string ProcessId, string ActivityName, string AttributeActionType, string sUser)
		{
			// Vallidate input
			if (AttributeActionType==null || AttributeActionType.Length==0)
				throw (new ArgumentOutOfRangeException("FormHelper - LoadXmlAttributes - No AttributeActionType"));

			// select witch type of load action on attributes to do
			switch (AttributeActionType)
			{
				case ACTION_PROCESSATTRIBUTES:
					// Vallidate input
					if (ProcessId==null || ProcessId.Length==0)
						throw (new ArgumentOutOfRangeException("FormHelper - LoadXmlAttributes - No ProcessId"));

					// load process attributes in module variable
					try
					{
						FH_ProcessAttributes = new XMLAttributes();
						FH_ProcessAttributes.Parse(this.GetProcessAttributes(ProcessId));
					}
					catch
					{
						FH_ProcessAttributes = null;
					}
					break;

				case ACTION_ACTIVITYATTRIBUTES:
					// Vallidate input
					if (ProcessId==null || ProcessId.Length==0)
						throw (new ArgumentOutOfRangeException("FormHelper - LoadXmlAttributes - No ProcessId"));
					
					if (ActivityName==null || ActivityName.Length==0)
						throw (new ArgumentOutOfRangeException("FormHelper - LoadXmlAttributes - No ActivityName"));

					// load activity attributes in module variable
					try
					{
						FH_ActivityAttributes = new XMLAttributes();
						FH_ActivityAttributes.Parse(this.GetActivityAttributes(ProcessId,ActivityName,sUser));
					}
					catch
					{
						FH_ProcessAttributes = null;
					}
					break;

				case ACTION_PCMENTITIES:
					// Vallidate input
					if (FH_ActivityAttributes==null || FH_ActivityAttributes.Count==0)
						throw (new ArgumentOutOfRangeException("FormHelper - LoadXmlAttributes - No PCM Entities"));
					
					// load PCM attributes in module variable
					try
					{
						FH_PCM_Entities = new XMLEntities ();
						FH_PCM_Entities.Parse(this.GetAttribute("PCM_Entities",ACTION_PCMENTITIES));
					}
					catch
					{
						FH_PCM_Entities = null;
					}
					
					;
					break;

				case ACTION_PCMATTRIBUTES:
					// Vallidate input
					if (FH_ProcessAttributes==null || FH_ProcessAttributes.Count==0)
						throw (new ArgumentOutOfRangeException("FormHelper - LoadXmlAttributes - No PCM Attributes"));
					
					// load PCM attributes in module variable
					try
					{
						FH_PCM_Attributes = new XMLEntities();
						FH_PCM_Attributes.Parse(this.GetAttribute("PCM_Attributes",ACTION_PCMATTRIBUTES));
					}
					catch
					{
						FH_PCM_Attributes = null;
					}
					
					;
					break;

			}
		}

		
		// ################################################################################################
		// ##
		// ## estract from FH_ActivityAttributes, FH_ProcessAttributes
		// ## FH_PCM_Entities, FH_PCM_Attributes.
		// ##
		// ################################################################################################
		public string GetAttribute(string Name, string AttributeActionType)
		{
			if (AttributeActionType==null || AttributeActionType.Length==0)
			{
				return string.Empty;
			}

			if (Name==null || Name.Length==0)
			{
				return string.Empty;
			}
					
			string AtributeValue = string.Empty ;

			// select witch type of load action on attributes to do
			switch (AttributeActionType)
			{
				case ACTION_PROCESSATTRIBUTES:

					if (FH_ProcessAttributes==null)
					{
						return string.Empty;
					}
						
					try
					{
						// search attribute in Module process atributes
						XMLAttribute attr = new XMLAttribute();
						attr = FH_ProcessAttributes.Get(Name);
						AtributeValue = attr.Value;
					}
					catch (ScenarioFramework.WorkflowEngine.DataModel.Exception.AttributeNotFound)
					{
						return string.Empty;
					}
					break;

				case ACTION_ACTIVITYATTRIBUTES:

					if (FH_ActivityAttributes==null)
					{
						return string.Empty;
					}

					try
					{
						// search attribute in Module activity atributes
						XMLAttribute attr = new XMLAttribute();
						attr = FH_ActivityAttributes.Get(Name);
						AtributeValue = attr.Value;
					}
                    catch (ScenarioFramework.WorkflowEngine.DataModel.Exception.AttributeNotFound)
					{
						return string.Empty;
					}
					break;

				case ACTION_PCMATTRIBUTES:

					if (FH_ProcessAttributes==null)
					{
						return string.Empty;
					}
					

					try
					{
						// search attribute in Module process atributes
						XMLAttribute attr = new XMLAttribute();
						attr = FH_ProcessAttributes.Get(Name);
						AtributeValue = attr.Value;
					}
                    catch (ScenarioFramework.WorkflowEngine.DataModel.Exception.AttributeNotFound)
					{
						return string.Empty;
					}
					break;

				case ACTION_PCMENTITIES:

					if (FH_ActivityAttributes==null)
					{
						return string.Empty;
					}

					try
					{
						// search attribute in Module activity atributes
						XMLAttribute attr = new XMLAttribute();
						attr = FH_ActivityAttributes.Get(Name);
						AtributeValue = attr.Value;
					}
                    catch (ScenarioFramework.WorkflowEngine.DataModel.Exception.AttributeNotFound)
					{
						return string.Empty;
					}
					break;

			}

			return AtributeValue;

		}

		// ################################################################################################
		// ##
		// ## Search inside PCM Attributes a single value for an EntityAttribute or an Entity by the
		// ## ExtendedEntityAttributePath like "EntityName.AtributeName" or "EntityName"
		// ##
		// ## Usually a FormHelper Istance calls this method after loading parameters form process into 
		// ## variables to extract previus values already configured into control visualization
		// ################################################################################################
		public string ExtractEntityAttributeValue(string ExtendedEntityAttributePath)
		{
			// Validate Input
			if (ExtendedEntityAttributePath==null || ExtendedEntityAttributePath.Length==0)
				throw (new ArgumentOutOfRangeException("FormHelper - ExtractEntityAttributeValue - No ExtendedEntityAttributePath"));
			
			if (FH_PCM_Attributes==null || FH_PCM_Attributes.Count==0)
				throw (new ArgumentOutOfRangeException("FormHelper - ExtractEntityAttributeValue - No PCM Attributes"));

			// Parse ExtendedEntityAttributePath

			// inizialize variables
			int enIDX=-1;
			//int atIDX=-1;
			string enNAME=string.Empty;
			string atNAME=string.Empty;

			// extract position and names from ExtendedEntityAttributePath
			enIDX = ExtendedEntityAttributePath.IndexOf(".",0);
			if (enIDX <= 0)
				enNAME = ExtendedEntityAttributePath;
			else
			{
				enNAME = ExtendedEntityAttributePath.Substring(0,(enIDX));

				//atIDX = ExtendedEntityAttributePath.IndexOf(".",enIDX+1);

				//if (atIDX <= enIDX+1)
				//	atNAME=string.Empty;
				//else
				atNAME = ExtendedEntityAttributePath.Substring(enIDX+1);
			}
			
			// extract entity Value or entity.attribute value

			for(int i=0; i<FH_PCM_Attributes.Count; i++)
			{
				XMLAttribute AuxAtt = (XMLAttribute) FH_PCM_Attributes[i];
				if (AuxAtt.Name == enNAME)
					return AuxAtt.Value;
			}

			if (atNAME!=string.Empty)
			{
				for(int i=0; i<FH_PCM_Attributes.Count; i++)
				{
					XMLAttribute AuxAtt = (XMLAttribute) FH_PCM_Attributes[i];
					if (AuxAtt.Name == enNAME + "" + atNAME)
						return AuxAtt.Value;
				}
			}
			else
			{
				return "NO PCM ATTRIBUTE - ATTRIBUTE NOT FOUND";
			}

			return "NO PCM ATTRIBUTE - ENTITY NOT FOUND";

		}
		
		// ################################################################################################
		// ##
		// ## Search inside PCM Entities a single value for a property by the
		// ## ExtendedPropertyPath like "EntityName.AtributeName.PropertiesGroup.PropertyName"
		// ##
		// ## Usually a FormHelper Istance calls this method after loading parameters form process into 
		// ## variables to extract graphic configuration for any control thath manages and to learn witch
		// ## type of sense assing to the value extracted from PCM Attributes (Value to visualize or Select to execute)
		// ################################################################################################
		public string ExtractPropertyValue(string ExtendedPropertyPath)
		{
			// Validate Input
			if (ExtendedPropertyPath==null || ExtendedPropertyPath.Length==0)
				throw (new ArgumentOutOfRangeException("FormHelper - ExtractPropertyValue - No ExtendedPropertyPath"));
			
			if (FH_PCM_Entities==null || FH_PCM_Entities.Count==0)
				throw (new ArgumentOutOfRangeException("FormHelper - ExtractPropertyValue - No PCM Entities"));

			// Parse ExtendedPropertyPath

			// inizialize variables
			int enIDX=-1;
			int atIDX=-1;
			int grIDX=-1;
			//int prIDX=-1;
			string enNAME=string.Empty;
			string atNAME=string.Empty;
			string grNAME=string.Empty;
			string prNAME=string.Empty;

			// extract position and names from ExtendedPropertyPath
			enIDX = ExtendedPropertyPath.IndexOf(".",0);
			if (enIDX <= 0)
				return "NO PCM ENTITY - ENTITY VALUE";
			
			enNAME = ExtendedPropertyPath.Substring(0,(enIDX));

			atIDX = ExtendedPropertyPath.IndexOf(".",enIDX+1);
			if (atIDX <= enIDX+1)
				return "NO PCM ENTITY - ATTRIBUTE VALUE";
			
			atNAME = ExtendedPropertyPath.Substring(enIDX+1,(atIDX-enIDX-1));

			grIDX = ExtendedPropertyPath.IndexOf(".",atIDX+1);
			if (grIDX <= atIDX+1)
				return "NO PCM ENTITY - PROPERTIESGROUP VALUE";
			
			grNAME = ExtendedPropertyPath.Substring(atIDX+1,(grIDX-atIDX-1));

			//prIDX = ExtendedPropertyPath.IndexOf(".",grIDX+1);
			//if (prIDX <= grIDX+1)
			//	return "NO PCM ENTITY - PROPERTY NAME VALUE";
			
			prNAME = ExtendedPropertyPath.Substring(grIDX+1);

			// search extracted values from PCMATTRIBUTES

			// inizialize entities
			XMLEntity en = new XMLEntity();
			XMLEntityAttribute at = new XMLEntityAttribute();
			XMLPropertiesGroup grpr = new XMLPropertiesGroup();
			XMLProperty pr = new XMLProperty();
			XMLPropertyValue vl = new XMLPropertyValue();

			// extract entity
			if (FH_PCM_Entities.AlmostInByName(enNAME))
			{
				en = FH_PCM_Entities.Get(enNAME);
			}
			else
			{
				return "NO PCM ENTITY - ENTITY NOT FOUND";
			}


			// epolore attribute
			if (en.XMLEntityAttributes.AlmostInByName(atNAME))
			{
				at = en.XMLEntityAttributes.Get(atNAME);
			}
			else
			{
				return "NO PCM ENTITY - ENTITY ATTRIBUTE NOT FOUND";
			}

			// epolore properties
			if (at.XMLProperties.AlmostInByName(grNAME))
			{
				grpr = at.XMLProperties.Get(grNAME);
			}
			else
			{
				return "NO PCM ENTITY - PROPERTIES GROUP NOT FOUND";
			}

			// epolore propertiesgroup
			if (grpr.AlmostInByName(prNAME))
			{
				pr = grpr.Get(prNAME);
			}
			else
			{
				return "NO PCM ENTITY - PROPERTY NOT FOUND";
			}

			// extract property value
			if (pr.XMLPropertyValues.Count>0)
			{
				vl = (XMLPropertyValue) pr.XMLPropertyValues[0];
			}
			else
			{
				return "NO PCM ENTITY - PROPERTY VALUES NOT FOUND";
			}

			if (vl.Value==null || vl.Value.Length==0)
			{
				return null;
			}
			else
			{
				return vl.Value;
			}

		}
        
		public bool SaveXmlAttributes(string ProcessId, string ActivityName, string sUser, string AttributeActionType)
		{
			// Vallidate input
			if (AttributeActionType==null || AttributeActionType.Length==0)
				throw (new ArgumentOutOfRangeException("FormHelper - SaveXmlAttributes - No AttributeActionType"));

			bool saved = true;

			// select witch type of load action on attributes to do
			switch (AttributeActionType)
			{
				case ACTION_PROCESSATTRIBUTES:
					// Vallidate input
					if (ProcessId==null || ProcessId.Length==0)
						throw (new ArgumentOutOfRangeException("FormHelper - SaveXmlAttributes - No ProcessId"));

					if (FH_ProcessAttributes==null)
						throw (new ArgumentOutOfRangeException("FormHelper - SaveXmlAttributes - No Process Attribiutes to Save"));

					// save process attributes from module variable to process instance
					try
					{
						this.SetProcessAttributes(ProcessId,  FH_ProcessAttributes);
					}
					catch
					{
						saved = false;
					}

					break;

				case ACTION_ACTIVITYATTRIBUTES:
					// Vallidate input
					if (ProcessId==null || ProcessId.Length==0)
						throw (new ArgumentOutOfRangeException("FormHelper - SaveXmlAttributes - No ProcessId"));
					
					if (ActivityName==null || ActivityName.Length==0)
						throw (new ArgumentOutOfRangeException("FormHelper - SaveXmlAttributes - No ActivityName"));

					if (FH_ActivityAttributes==null)
						throw (new ArgumentOutOfRangeException("FormHelper - SaveXmlAttributes - No Activiry Attribiutes to Save"));
						

					// save activity attributes from module variable to activity
					try
					{
						this.SetActivityAttributes(ProcessId, ActivityName, sUser, FH_ActivityAttributes);
					}
					catch
					{
						saved = false;
					}
					break;

				case ACTION_PCMATTRIBUTES:
					// Vallidate input
					if (ProcessId==null || ProcessId.Length==0)
						throw (new ArgumentOutOfRangeException("FormHelper - SaveXmlAttributes - No ProcessId"));

					if (FH_ProcessAttributes==null)
						throw (new ArgumentOutOfRangeException("FormHelper - SaveXmlAttributes - No Process Attribiutes to Save"));

					if (FH_PCM_Attributes==null)
						throw (new ArgumentOutOfRangeException("FormHelper - SaveXmlAttributes - No PCM Attribiutes to Save"));
					
					// save process attributes from module variable to process instance
					try
					{
						// build attribute to set in process attributes
						XMLAttribute PCM_Attributes = new XMLAttribute();
						PCM_Attributes.Name="PCM_Attributes";
						PCM_Attributes.Type="XML";
						PCM_Attributes.Value=FH_PCM_Attributes.GetShortXML();
						//PCM_Attributes.Value=FH_PCM_Attributes.GetXML();
						
						this.SetAttribute(FormHelper.ACTION_PCMATTRIBUTES,PCM_Attributes.Value,FormHelper.ACTION_PROCESSATTRIBUTES);
					}
					catch
					{
						saved = false;
					}

					break;

				case ACTION_PCMENTITIES:
					// NO ACTIONS ON SAVE ATTRIBTES FOR PCM ENTITIES
					saved = false;
					break;
	
			}

			// return unmodified value true or modified value false
			return saved;
		}

		// ################################################################################################
		// ##
		// ## Set into FH_ActivityAttributes, FH_ProcessAttributes a single value for an attribute
		// ##
		// ################################################################################################
		public bool SetAttribute(string Name, string Value, string AttributeActionType)
		{
			// Vallidate input
			if (AttributeActionType==null || AttributeActionType.Length==0)
				throw (new ArgumentOutOfRangeException("FormHelper - SetAttribute - No AttributeActionType"));
            
			bool done = true;

			// select witch type of load action on attributes to do
			switch (AttributeActionType)
			{
				case ACTION_PROCESSATTRIBUTES:
					try
					{
						// search attribute in Module process atributes
						XMLAttribute attr = new XMLAttribute();
						attr = FH_ProcessAttributes.Get(Name);
						attr.Value = Value;
					}
                    catch (ScenarioFramework.WorkflowEngine.DataModel.Exception.AttributeNotFound)
					{
						done = false;
					}
					break;

				case ACTION_ACTIVITYATTRIBUTES:
					try
					{
						// search attribute in Module activity atributes
						XMLAttribute attr = new XMLAttribute();
						attr = FH_ActivityAttributes.Get(Name);
						attr.Value = Value;
					}
                    catch (ScenarioFramework.WorkflowEngine.DataModel.Exception.AttributeNotFound)
					{
						done = false;
					}
					break;

				case ACTION_PCMATTRIBUTES:
					try
					{
						FH_PCM_Attributes.SetEntityAttributeValue(Name, Value);
					}
                    catch (ScenarioFramework.WorkflowEngine.DataModel.Exception.AttributeNotFound)
					{
						done = false;
					}
					break;

				case ACTION_PCMENTITIES:
					// NO ACTIONS ON SET ATTRIBTES FOR PCM ATTRIBUTES
					done = false;
					break;

			}

			return done;
		}

		public string GetUrl(string sAction)
		{
			XMLAction action = null;
			try
			{
				action = allowedActions.Get(sAction);
				return action.RedirectUrl;
					
			}
			catch ( Exception.XMLActionNotFound )
			{}

			return string.Empty;
		}

		private string SetActivityAttributes (string ProcessId, string ActivityName, string sUser, XMLAttributes Attributes)
		{
			try 
			{
				// Compose XMLAcivity
				XMLActivity Activity = new XMLActivity();
				Activity.ProcessId = Int32.Parse(ProcessId);
				Activity.Name = ActivityName;

				// call command to do
				WAPI.Response response = new WAPI.Response();
				WAPI.Activity.SetActivityAttributes(Activity.GetXML(),Attributes.GetXML(), sUser, ref response);

				// return response
				if (response.iCode == 0)
					return "OK";
				else
					return response.sMsg;
			}
			catch
				// Parameters error
			{
				return "WAPI SetActivityAttributes - error";
			}

		}

		// ################################################################################################
		// ##
		// ## execute set attribute on process using FH_ProcessAttributes like source
		// ##
		// ################################################################################################
		private string SetProcessAttributes (string ProcessId, XMLAttributes Attributes)
		{
			try 
			{
				// call command to do
				WAPI.Response response = new WAPI.Response();
				//WAPI.Process.SetProcessInstanceAttributes(Attributes.GetXML(), Int32.Parse(ProcessId),ref response);
				WAPI.Process.UpdateProcessInstanceAttributes(Attributes.GetXML(), Int32.Parse(ProcessId),ref response);

				// return response
				if (response.iCode == 0)
					return "OK";
				else
					return response.sMsg;
			}
			catch
				// Parameters error
			{
				return "WAPI SetProcessAttributes - error";
			}

		}

		// ################################################################################################
		// ##
		// ## execute Get attribute on Activity 
		// ##
		// ################################################################################################
		private string GetActivityAttributes(string ProcessId, string ActivityName)
		{
			// Compose XMLAcivity
			XMLActivity Activity = new XMLActivity();
			Activity.ProcessId = Int32.Parse(ProcessId);
			Activity.Name = ActivityName;

			// call command to do
			WAPI.Response response = new WAPI.Response();
			string Attributes = WAPI.Activity.GetActivityAttributes(Activity.GetXML(), "", ref response);

			// return response
			if (response.iCode == 0)
				return Attributes;
			else
				return response.sMsg;
		}
		private string GetActivityAttributes(string ProcessId, string ActivityName, string sUser)
		{
			// Compose XMLAcivity
			XMLActivity Activity = new XMLActivity();
			Activity.ProcessId = Int32.Parse(ProcessId);
			Activity.Name = ActivityName;

			// call command to do
			WAPI.Response response = new WAPI.Response();
			string Attributes = WAPI.Activity.GetActivityAttributes(Activity.GetXML(), sUser, ref response);

			// return response
			if (response.iCode == 0)
				return Attributes;
			else
				return response.sMsg;
		}

		// ################################################################################################
		// ##
		// ## execute Get attribute on process 
		// ##
		// ################################################################################################
		private string GetProcessAttributes(string ProcessId)
		{
			// call command to do
			WAPI.Response response = new WAPI.Response();
			string Attributes = WAPI.Process.GetProcessInstanceAttributes(ref response, Int32.Parse(ProcessId));

			// return response
			if (response.iCode == 0)
				return Attributes;
			else
				return response.sMsg;
		}

		// ################################################################################################
		// ##
		// ## extract parameter value to return redirect url 
		// ## if a redirect url is a sintiax like [param] it search paramter value and returs it
		// ##
		// ################################################################################################
		private string GetRedirectParameter (string RedUrl)
		{
			//validate input
			if (!((RedUrl == null) || (RedUrl.Length ==0)))
			{
				// check if there are squre parentesis in redirect url to evaluate if is a parameter
				int start = -1;
				start = RedUrl.IndexOf("[",0);
				if (start >=0)
				{
					int end = -1;
					end = RedUrl.IndexOf("]",start+1);
					if ((end >=0) && (end > start))
					{
						// estract parameter name
						string Param = RedUrl.Substring((start+1),(end-start-1));
												
						// search parameter into activity attributes collection
						string URL = GetAttribute(Param, FormHelper.ACTION_ACTIVITYATTRIBUTES);

						if (URL!= null)
							return URL;
						else
						{
							// search parameter into process attributes collection
							URL = GetAttribute(Param, FormHelper.ACTION_PROCESSATTRIBUTES);
							if (URL!= null)
								return URL;
							else
								return "Return url - Parameter value not found";
						}
					}
					else
					{
						return RedUrl;
					}
				}
				else
				{
					return RedUrl;
				}

			}
			else
				return null;
		}

		// ################################################################################################
		// ##
		// ## execute method select from actions a single action to execute and does it
		// ##
		// ################################################################################################
		public string ExecuteAction(string User, string ActionName, string ProcessId, string ActivityName)
		{
			// Vallidate input
			if (ActionName==null || ActionName.Length==0)
			{
				throw (new ArgumentOutOfRangeException("FormHelper - No ActionName"));
			}

			// find action from allowed colletion
			XMLAction action;
			try
			{
				action = allowedActions.Get(ActionName);
			}
			catch(Exception.XMLActionNotFound)
			{
				return "NO ACTION ALLOWED";
			}

			// parse action to do
			string ReturnedResponse = "OK";
			bool bNotErr=true;

			// check if there are flags on activity action
			if ( action.ProcessAction.ActivityAction.vFlags.Count > 0)
			{
				// Vallidate input
				if (ProcessId==null || ProcessId.Length==0)
					throw (new ArgumentOutOfRangeException("FormHelper - No ProcessId"));

				if (ActivityName==null || ActivityName.Length==0)
					throw (new ArgumentOutOfRangeException("FormHelper - No ActivityName"));

				XMLFlag actionFlag = new XMLFlag();

				/** find the sub tags witch contains flags values on activity action **/
				
				for ( int i=0; i<action.ProcessAction.ActivityAction.vFlags.Count && bNotErr; i++)
				{
					actionFlag = (XMLFlag) action.ProcessAction.ActivityAction.vFlags[i];

					// select for each flag the action type to do on activity
					switch (actionFlag.sType)
					{
					
						case  ACTION_ACTIVITYATTRIBUTES:
							
							// select witch type of activity attributes action do it have to do
						switch (actionFlag.sValue)
						{
							case  XMLFlag.SET_VALUE:
								this.SaveXmlAttributes(ProcessId,ActivityName,User,FormHelper.ACTION_ACTIVITYATTRIBUTES);
								break;
									
							case  XMLFlag.GET_VALUE:
								this.LoadXmlAttributes(ProcessId,ActivityName,FormHelper.ACTION_ACTIVITYATTRIBUTES);
								break;
						}
							
							break;

						case ACTION_ACTIVITYDO:

							// select witch type of activity attributes action do it have to do
						switch (actionFlag.sValue)
						{
							case  XMLFlag.COMPLETE_TYPE:
							{
								// Validate input 
								if (User==null || User.Length==0)
									throw (new ArgumentOutOfRangeException("FormHelper - No User"));

								// Compose XMLAcivity
								XMLActivity Activity = new XMLActivity();
								Activity.ProcessId = Int32.Parse(ProcessId);
								Activity.Name = ActivityName;

								// call command to do
								WAPI.Response response = new WAPI.Response();
								String sActivityLog = WAPI.Activity.GetActivityLog (Activity.GetXML(),User,ref response);
								XMLLog xmlLog  = new XMLLog();
								xmlLog.Parse (sActivityLog);
								if ( xmlLog.Status.CompareTo ( XMLLog.RUNNING_STATUS ) == 0 ) 
								{
									WAPI.WorkItem.CompleteWorkItem(Activity.GetXML(),User, "OK", ref response);
									if (response.iCode != 0)
									{
										ReturnedResponse = "KO";
									}
								}
								break;
							}
								
							case  XMLFlag.RELEASE_TYPE:
							{
								// Compose XMLAcivity
								XMLActivity Activity = new XMLActivity();
								Activity.ProcessId = Int32.Parse(ProcessId);
								Activity.Name = ActivityName;

								// call command to do
								WAPI.Response response = new WAPI.Response();
								WAPI.WorkItem.ReleaseWorkItem(Activity.GetXML(), ref response);
								if (response.iCode != 0)
									ReturnedResponse = "KO";
								break;
							}

							case  XMLFlag.REASSIGN_TYPE:
							{
								// Validate input 
								if (User==null || User.Length==0)
									throw (new ArgumentOutOfRangeException("FormHelper - No User"));

								// Compose XMLAcivity
								XMLActivity Activity = new XMLActivity();
								Activity.ProcessId = Int32.Parse(ProcessId);
								Activity.Name = ActivityName;

								// call command to do
								WAPI.Response response = new WAPI.Response();
								WAPI.WorkItem.ReassignWorkItem(Activity.GetXML(), User, ref response);
								if (response.iCode != 0)
									ReturnedResponse = "KO";
								break;
							}

							case  XMLFlag.STOP_TYPE:
							{
								// Validate input 
								if (User==null || User.Length==0)
									throw (new ArgumentOutOfRangeException("FormHelper - No User"));

								// Compose XMLAcivity
								XMLActivity Activity = new XMLActivity();
								Activity.ProcessId = Int32.Parse(ProcessId);
								Activity.Name = ActivityName;

								// call command to do
								WAPI.Response response = new WAPI.Response();
								WAPI.WorkItem.StopWorkItem(Activity.GetXML(), User, ref response);
								if (response.iCode != 0)
									ReturnedResponse = "KO";
								break;
							}

							case  XMLFlag.DO_TYPE:
							{
								// Validate input 
								if (User==null || User.Length==0)
									throw (new ArgumentOutOfRangeException("FormHelper - No User"));

								// Compose XMLAcivity
								XMLActivity Activity = new XMLActivity();
								Activity.ProcessId = Int32.Parse(ProcessId);
								Activity.Name = ActivityName;

								// call command to do
								WAPI.Response response = new WAPI.Response();
								WAPI.WorkItem.DoWorkItem(Activity.GetXML(), User, ref response);
								if (response.iCode != 0)
									ReturnedResponse = "KO";
								break;
							}

							case XMLFlag.ACQUIRE_TYPE:
							{
								// Validate input 
								if (User==null || User.Length==0)
									throw (new ArgumentOutOfRangeException("FormHelper - No User"));

								// Compose XMLAcivity
								XMLActivity Activity = new XMLActivity();
								Activity.ProcessId = Int32.Parse(ProcessId);
								Activity.Name = ActivityName;

								// call command to do
								WAPI.Response response = new WAPI.Response();
								WAPI.Activity.AcquireActivity(Activity.GetXML(),User,ref response);
								if (response.iCode != 0)
								{
									bNotErr=false;
									ReturnedResponse = "KO";
								}
								break;
							}

							case XMLFlag.ASSIGN_TYPE:
							{
								// Validate input 
								if (User==null || User.Length==0)
									throw (new ArgumentOutOfRangeException("FormHelper - No User"));

								// Compose XMLAcivity
								XMLActivity Activity = new XMLActivity();
								Activity.ProcessId = Int32.Parse(ProcessId);
								Activity.Name = ActivityName;

								// call command to do
								WAPI.Response response = new WAPI.Response();
								WAPI.Activity.AssignActivity(Activity.GetXML(),User,ref response);
								if (response.iCode != 0)
									ReturnedResponse = "KO";
								break;
							}

						}

							break;
					}
				}
			}

			// check if there are flags on process action
			if ( action.ProcessAction.vFlags.Count > 0)
			{
				// Vallidate input
				if (ProcessId==null || ProcessId.Length==0)
					throw (new ArgumentOutOfRangeException("FormHelper - No ProcessId"));


				XMLFlag actionFlag = new XMLFlag();

				/** find the sub tags witch contains flags values on activity action **/
				for ( int i=0; i<action.ProcessAction.vFlags.Count; i++)
				{
					actionFlag = (XMLFlag) action.ProcessAction.vFlags[i];

					// select for each flag the action type to do on activity
					switch (actionFlag.sType)
					{
					
						case  ACTION_PROCESSATTRIBUTES:
							
							// select witch type of activity attributes action do it have to do
						switch (actionFlag.sValue)
						{
							case  XMLFlag.SET_VALUE:
								this.SaveXmlAttributes(ProcessId,ActivityName,User,FormHelper.ACTION_PROCESSATTRIBUTES);
								break;
										
							case  XMLFlag.GET_VALUE:
								this.LoadXmlAttributes(ProcessId,ActivityName,FormHelper.ACTION_PROCESSATTRIBUTES);
								break;
						}
							
							break;
					}
				}
			}

			// return the associated redirect url
			if ( bNotErr )
			{
				if (!((action.RedirectUrl == null) || (action.RedirectUrl.Length ==0)))
				{
					ReturnedResponse = this.GetRedirectParameter(action.RedirectUrl);
				}
			}
			else
			{}

			return ReturnedResponse;
		}
		public string ExecuteActionKO(string User, string ActionName, string ProcessId, string ActivityName)
		{
			// Vallidate input
			if (ActionName==null || ActionName.Length==0)
			{
				throw (new ArgumentOutOfRangeException("FormHelper - No ActionName"));
			}

			// find action from allowed colletion
			XMLAction action;
			try
			{
				action = allowedActions.Get(ActionName);
			}
			catch(Exception.XMLActionNotFound)
			{
				if ( ProcessId != "" && ActivityName != "")
				{
					XMLActivity activity = new XMLActivity();
					activity.ProcessId = System.Int32.Parse (ProcessId);
					activity.Name = ActivityName;

					WAPI.Response wapiResponse = new WAPI.Response();
					string sAttributes = WAPI.WorkItem.GetWorkItemAttributes (activity.GetXML(), User, ref wapiResponse);

					if ( wapiResponse.iCode == 0 )
					{
						activity.XMLAttributes.Parse (sAttributes);

						XMLAttribute formAttribute = (XMLAttribute)activity.XMLAttributes.Get("Form");
						if ( formAttribute != null )
						{
							return formAttribute.Value;
						}
					}
				}
				return "NO ACTION ALLOWED";
			}

			// parse action to do
			string ReturnedResponse = "OK";
			bool bNotErr=true;

			// check if there are flags on activity action
			if ( action.ProcessAction.ActivityAction.vFlags.Count > 0)
			{
				// Vallidate input
				if (ProcessId==null || ProcessId.Length==0)
					throw (new ArgumentOutOfRangeException("FormHelper - No ProcessId"));

				if (ActivityName==null || ActivityName.Length==0)
					throw (new ArgumentOutOfRangeException("FormHelper - No ActivityName"));

				XMLFlag actionFlag = new XMLFlag();

				/** find the sub tags witch contains flags values on activity action **/
				
				for ( int i=0; i<action.ProcessAction.ActivityAction.vFlags.Count && bNotErr; i++)
				{
					actionFlag = (XMLFlag) action.ProcessAction.ActivityAction.vFlags[i];

					// select for each flag the action type to do on activity
					switch (actionFlag.sType)
					{
					
						case  ACTION_ACTIVITYATTRIBUTES:
							
							// select witch type of activity attributes action do it have to do
						switch (actionFlag.sValue)
						{
							case  XMLFlag.SET_VALUE:
								this.SaveXmlAttributes(ProcessId,ActivityName,User,FormHelper.ACTION_ACTIVITYATTRIBUTES);
								break;
									
							case  XMLFlag.GET_VALUE:
								this.LoadXmlAttributes(ProcessId,ActivityName,FormHelper.ACTION_ACTIVITYATTRIBUTES);
								break;
						}
							
							break;

						case ACTION_ACTIVITYDO:

							// select witch type of activity attributes action do it have to do
						switch (actionFlag.sValue)
						{
							case  XMLFlag.COMPLETE_TYPE:
							{
								// Validate input 
								if (User==null || User.Length==0)
									throw (new ArgumentOutOfRangeException("FormHelper - No User"));

								// Compose XMLAcivity
								XMLActivity Activity = new XMLActivity();
								Activity.ProcessId = Int32.Parse(ProcessId);
								Activity.Name = ActivityName;

								// call command to do
								WAPI.Response response = new WAPI.Response();
								WAPI.WorkItem.CompleteWorkItem(Activity.GetXML(),User, "KO", ref response);
								if (response.iCode != 0)
								{
									ReturnedResponse = "KO";
								}
								break;
							}
								
							case  XMLFlag.RELEASE_TYPE:
							{
								// Compose XMLAcivity
								XMLActivity Activity = new XMLActivity();
								Activity.ProcessId = Int32.Parse(ProcessId);
								Activity.Name = ActivityName;

								// call command to do
								WAPI.Response response = new WAPI.Response();
								WAPI.WorkItem.ReleaseWorkItem(Activity.GetXML(), ref response);
								if (response.iCode != 0)
									ReturnedResponse = "KO";
								break;
							}

							case  XMLFlag.REASSIGN_TYPE:
							{
								// Validate input 
								if (User==null || User.Length==0)
									throw (new ArgumentOutOfRangeException("FormHelper - No User"));

								// Compose XMLAcivity
								XMLActivity Activity = new XMLActivity();
								Activity.ProcessId = Int32.Parse(ProcessId);
								Activity.Name = ActivityName;

								// call command to do
								WAPI.Response response = new WAPI.Response();
								WAPI.WorkItem.ReassignWorkItem(Activity.GetXML(), User, ref response);
								if (response.iCode != 0)
									ReturnedResponse = "KO";
								break;
							}

							case  XMLFlag.STOP_TYPE:
							{
								// Validate input 
								if (User==null || User.Length==0)
									throw (new ArgumentOutOfRangeException("FormHelper - No User"));

								// Compose XMLAcivity
								XMLActivity Activity = new XMLActivity();
								Activity.ProcessId = Int32.Parse(ProcessId);
								Activity.Name = ActivityName;

								// call command to do
								WAPI.Response response = new WAPI.Response();
								WAPI.WorkItem.StopWorkItem(Activity.GetXML(), User, ref response);
								if (response.iCode != 0)
									ReturnedResponse = "KO";
								break;
							}

							case  XMLFlag.DO_TYPE:
							{
								// Validate input 
								if (User==null || User.Length==0)
									throw (new ArgumentOutOfRangeException("FormHelper - No User"));

								// Compose XMLAcivity
								XMLActivity Activity = new XMLActivity();
								Activity.ProcessId = Int32.Parse(ProcessId);
								Activity.Name = ActivityName;

								// call command to do
								WAPI.Response response = new WAPI.Response();
								WAPI.WorkItem.DoWorkItem(Activity.GetXML(), User, ref response);
								if (response.iCode != 0)
									ReturnedResponse = "KO";
								break;
							}

							case XMLFlag.ACQUIRE_TYPE:
							{
								// Validate input 
								if (User==null || User.Length==0)
									throw (new ArgumentOutOfRangeException("FormHelper - No User"));

								// Compose XMLAcivity
								XMLActivity Activity = new XMLActivity();
								Activity.ProcessId = Int32.Parse(ProcessId);
								Activity.Name = ActivityName;

								// call command to do
								WAPI.Response response = new WAPI.Response();
								WAPI.Activity.AcquireActivity(Activity.GetXML(),User,ref response);
								if (response.iCode != 0)
								{
									bNotErr=false;
									ReturnedResponse = "KO";
								}
								break;
							}

							case XMLFlag.ASSIGN_TYPE:
							{
								// Validate input 
								if (User==null || User.Length==0)
									throw (new ArgumentOutOfRangeException("FormHelper - No User"));

								// Compose XMLAcivity
								XMLActivity Activity = new XMLActivity();
								Activity.ProcessId = Int32.Parse(ProcessId);
								Activity.Name = ActivityName;

								// call command to do
								WAPI.Response response = new WAPI.Response();
								WAPI.Activity.AssignActivity(Activity.GetXML(),User,ref response);
								if (response.iCode != 0)
									ReturnedResponse = "KO";
								break;
							}

						}

							break;
					}
				}
			}

			// check if there are flags on process action
			if ( action.ProcessAction.vFlags.Count > 0)
			{
				// Vallidate input
				if (ProcessId==null || ProcessId.Length==0)
					throw (new ArgumentOutOfRangeException("FormHelper - No ProcessId"));


				XMLFlag actionFlag = new XMLFlag();

				/** find the sub tags witch contains flags values on activity action **/
				for ( int i=0; i<action.ProcessAction.vFlags.Count; i++)
				{
					actionFlag = (XMLFlag) action.ProcessAction.vFlags[i];

					// select for each flag the action type to do on activity
					switch (actionFlag.sType)
					{
					
						case  ACTION_PROCESSATTRIBUTES:
							
							// select witch type of activity attributes action do it have to do
						switch (actionFlag.sValue)
						{
							case  XMLFlag.SET_VALUE:
								this.SaveXmlAttributes(ProcessId,ActivityName,User,FormHelper.ACTION_PROCESSATTRIBUTES);
								break;
										
							case  XMLFlag.GET_VALUE:
								this.LoadXmlAttributes(ProcessId,ActivityName,FormHelper.ACTION_PROCESSATTRIBUTES);
								break;
						}
							
							break;
					}
				}
			}

			// return the associated redirect url
			if ( bNotErr )
			{
				if (!((action.RedirectUrl == null) || (action.RedirectUrl.Length ==0)))
				{
					ReturnedResponse = this.GetRedirectParameter(action.RedirectUrl);
				}
			}
			else
			{}

			return ReturnedResponse;
		}


		//Metodo Implementato da Enza.
		public string ExecuteComplete(string User,string ActionName, string ProcessId, string ActivityName)
		{
			// Vallidate input
			if (ActionName==null || ActionName.Length==0)
			{
				throw (new ArgumentOutOfRangeException("FormHelper - No ActionName"));
			}

			// find action from allowed colletion
			XMLAction action;
			try
			{
				action = allowedActions.Get(ActionName);
			}
			catch(Exception.XMLActionNotFound)
			{
				if ( ProcessId != "" && ActivityName != "")
				{
					XMLActivity activity = new XMLActivity();
					activity.ProcessId = System.Int32.Parse (ProcessId);
					activity.Name = ActivityName;

					WAPI.Response wapiResponse = new WAPI.Response();
					string sAttributes = WAPI.WorkItem.GetWorkItemAttributes (activity.GetXML(), User, ref wapiResponse);

					if ( wapiResponse.iCode == 0 )
					{
						activity.XMLAttributes.Parse (sAttributes);
						XMLAttribute formAttribute = (XMLAttribute)activity.XMLAttributes.Get("Form");
						if ( formAttribute != null )
						{
							return formAttribute.Value;
						}
					}
				}
				return "NO ACTION ALLOWED";
			}

			// parse action to do
			string ReturnedResponse = "OK";
			bool bNotErr=true;

			// check if there are flags on activity action
			if ( action.ProcessAction.ActivityAction.vFlags.Count > 0)
			{
				// Vallidate input
				if (ProcessId==null || ProcessId.Length==0)
					throw (new ArgumentOutOfRangeException("FormHelper - No ProcessId"));

				if (ActivityName==null || ActivityName.Length==0)
					throw (new ArgumentOutOfRangeException("FormHelper - No ActivityName"));

				XMLFlag actionFlag = new XMLFlag();
				XMLReturnValue returnValue= new XMLReturnValue();

				/** find the sub tags witch contains flags values on activity action **/
				
				for ( int i=0; i<action.ProcessAction.ActivityAction.vFlags.Count && bNotErr; i++)
				{
					actionFlag = (XMLFlag) action.ProcessAction.ActivityAction.vFlags[i];

					// select for each flag the action type to do on activity
					switch (actionFlag.sType)
					{
					
						case  ACTION_ACTIVITYATTRIBUTES:
							
							// select witch type of activity attributes action do it have to do
						switch (actionFlag.sValue)
						{
							case  XMLFlag.SET_VALUE:
								this.SaveXmlAttributes(ProcessId,ActivityName,User,FormHelper.ACTION_ACTIVITYATTRIBUTES);
								break;
									
							case  XMLFlag.GET_VALUE:
								this.LoadXmlAttributes(ProcessId,ActivityName,FormHelper.ACTION_ACTIVITYATTRIBUTES);
								break;
						}
							
							break;

						case ACTION_ACTIVITYDO:

							// select witch type of activity attributes action do it have to do
						switch (actionFlag.sValue)
						{
							case  XMLFlag.COMPLETE_TYPE:
							{
								
								// Validate input 
								string exitStatus=string.Empty;
								if (User==null || User.Length==0)
									throw (new ArgumentOutOfRangeException("FormHelper - No User"));
								
								// Compose XMLAcivity
								XMLActivity Activity = new XMLActivity();
								Activity.ProcessId = Int32.Parse(ProcessId);
								Activity.Name = ActivityName;
								returnValue= (XMLReturnValue) action.ProcessAction.ActivityAction.vReturnValue;

								// call command to do
								WAPI.Response response = new WAPI.Response();

								XmlDocument doc=new XmlDocument();
								this.LoadXmlAttributes(ProcessId,ActivityName,"ProcessAttributes");
								string docId=GetAttribute(returnValue.ToTransform,"ProcessAttributes");
															
								DocumentManagerDBHelper  dmDBHelper = new DocumentManagerDBHelper();
								DocumentHelper docHelper=new DocumentHelper (dmDBHelper);
								string sFileName= docHelper.GetInstanceFileName(docId ,"1",docId,"xml");
								if(returnValue.Type==XMLReturnValue.XSLT_VALUE)
								{
								
									XmlTextReader tr11=new XmlTextReader(sFileName);
									XPathDocument xPathDocument=new XPathDocument(tr11);
									XslTransform xslt = new XslTransform ();
			
									string nomexsl=returnValue.Value +".xsl";
									xslt.Load(nomexsl);
    
									//create the output stream
									StringBuilder sb=new StringBuilder();
									TextWriter tw=new StringWriter(sb);

									xslt.Transform(xPathDocument,null, tw,null);
									
									exitStatus=tw.ToString();	
										
								}
								else if (returnValue.Type== XMLReturnValue.XPATH_VALUE)
								{
									
									doc.Load(sFileName);
									XmlNode node= doc.SelectSingleNode(returnValue.Value,null);
									if (node!= null)
									{
										if (node.InnerText=="uno")
											exitStatus=node.InnerText;
									}
									else exitStatus= "KO";
										
								}
								
								WAPI.WorkItem.CompleteWorkItem(Activity.GetXML(),User, exitStatus, ref response);
								if (response.iCode != 0)
								{
									ReturnedResponse = "KO";
								}
								break;
							}
								
							case  XMLFlag.RELEASE_TYPE:
							{
								// Compose XMLAcivity
								XMLActivity Activity = new XMLActivity();
								Activity.ProcessId = Int32.Parse(ProcessId);
								Activity.Name = ActivityName;

								// call command to do
								WAPI.Response response = new WAPI.Response();
								WAPI.WorkItem.ReleaseWorkItem(Activity.GetXML(), ref response);
								if (response.iCode != 0)
									ReturnedResponse = "KO";
								break;
							}

							case  XMLFlag.REASSIGN_TYPE:
							{
								// Validate input 
								if (User==null || User.Length==0)
									throw (new ArgumentOutOfRangeException("FormHelper - No User"));

								// Compose XMLAcivity
								XMLActivity Activity = new XMLActivity();
								Activity.ProcessId = Int32.Parse(ProcessId);
								Activity.Name = ActivityName;

								// call command to do
								WAPI.Response response = new WAPI.Response();
								WAPI.WorkItem.ReassignWorkItem(Activity.GetXML(), User, ref response);
								if (response.iCode != 0)
									ReturnedResponse = "KO";
								break;
							}

							case  XMLFlag.STOP_TYPE:
							{
								// Validate input 
								if (User==null || User.Length==0)
									throw (new ArgumentOutOfRangeException("FormHelper - No User"));

								// Compose XMLAcivity
								XMLActivity Activity = new XMLActivity();
								Activity.ProcessId = Int32.Parse(ProcessId);
								Activity.Name = ActivityName;

								// call command to do
								WAPI.Response response = new WAPI.Response();
								WAPI.WorkItem.StopWorkItem(Activity.GetXML(), User, ref response);
								if (response.iCode != 0)
									ReturnedResponse = "KO";
								break;
							}

							case  XMLFlag.DO_TYPE:
							{
								// Validate input 
								if (User==null || User.Length==0)
									throw (new ArgumentOutOfRangeException("FormHelper - No User"));

								// Compose XMLAcivity
								XMLActivity Activity = new XMLActivity();
								Activity.ProcessId = Int32.Parse(ProcessId);
								Activity.Name = ActivityName;

								// call command to do
								WAPI.Response response = new WAPI.Response();
								WAPI.WorkItem.DoWorkItem(Activity.GetXML(), User, ref response);
								if (response.iCode != 0)
									ReturnedResponse = "KO";
								break;
							}

							case XMLFlag.ACQUIRE_TYPE:
							{
								// Validate input 
								if (User==null || User.Length==0)
									throw (new ArgumentOutOfRangeException("FormHelper - No User"));

								// Compose XMLAcivity
								XMLActivity Activity = new XMLActivity();
								Activity.ProcessId = Int32.Parse(ProcessId);
								Activity.Name = ActivityName;

								// call command to do
								WAPI.Response response = new WAPI.Response();
								WAPI.Activity.AcquireActivity(Activity.GetXML(),User,ref response);
								if (response.iCode != 0)
								{
									bNotErr=false;
									ReturnedResponse = "KO";
								}
								break;
							}

							case XMLFlag.ASSIGN_TYPE:
							{
								// Validate input 
								if (User==null || User.Length==0)
									throw (new ArgumentOutOfRangeException("FormHelper - No User"));

								// Compose XMLAcivity
								XMLActivity Activity = new XMLActivity();
								Activity.ProcessId = Int32.Parse(ProcessId);
								Activity.Name = ActivityName;

								// call command to do
								WAPI.Response response = new WAPI.Response();
								WAPI.Activity.AssignActivity(Activity.GetXML(),User,ref response);
								if (response.iCode != 0)
									ReturnedResponse = "KO";
								break;
							}

						}

							break;
					}
				}
			}

			// check if there are flags on process action
			if ( action.ProcessAction.vFlags.Count > 0)
			{
				// Vallidate input
				if (ProcessId==null || ProcessId.Length==0)
					throw (new ArgumentOutOfRangeException("FormHelper - No ProcessId"));


				XMLFlag actionFlag = new XMLFlag();

				/** find the sub tags witch contains flags values on activity action **/
				for ( int i=0; i<action.ProcessAction.vFlags.Count; i++)
				{
					actionFlag = (XMLFlag) action.ProcessAction.vFlags[i];

					// select for each flag the action type to do on activity
					switch (actionFlag.sType)
					{
					
						case  ACTION_PROCESSATTRIBUTES:
							
							// select witch type of activity attributes action do it have to do
						switch (actionFlag.sValue)
						{
							case  XMLFlag.SET_VALUE:
								this.SaveXmlAttributes(ProcessId,ActivityName,User,FormHelper.ACTION_PROCESSATTRIBUTES);
								break;
										
							case  XMLFlag.GET_VALUE:
								this.LoadXmlAttributes(ProcessId,ActivityName,FormHelper.ACTION_PROCESSATTRIBUTES);
								break;
						}
							
							break;
					}
				}
			}

			// return the associated redirect url
			if ( bNotErr )
			{
				if (!((action.RedirectUrl == null) || (action.RedirectUrl.Length ==0)))
				{
					ReturnedResponse = this.GetRedirectParameter(action.RedirectUrl);
				}
			}
			else
			{}

			return ReturnedResponse;
		}
		private string readTag(string sXpathStatement, XmlDocument doc)
		{
			if (sXpathStatement!=string.Empty)
			{
				sXpathStatement = resolveAZSquare(sXpathStatement);
				XmlNode node=doc.SelectSingleNode(sXpathStatement,null);
				if (node!= null)
				{
					return node.InnerText;
				}
				return null;
			}
			return string.Empty;
		}
		private void writeTag(string sXpathStatement, XmlDocument doc, string sInnerText)
		{
			if (sXpathStatement!=string.Empty)
			{
				sXpathStatement = resolveAZSquare(sXpathStatement);

				//NameSpace
				string sDocumentName = doc.DocumentElement.Name;

				try
				{
					XMLEnumerationValues vValues = WAPI.DataTypes.GetValues ( sDocumentName + "_Namespaces");
					XmlNamespaceManager manager = new XmlNamespaceManager(doc.NameTable);
					XMLDataTypeValue vValue = null;
					for ( int i=0; i<vValues.Count;i++)
					{
						vValue = (XMLDataTypeValue)vValues[i];
						manager.AddNamespace (vValue.Key , vValue.Value);
					}
					XmlNode node=doc.SelectSingleNode(sXpathStatement,manager);

					if (node!= null)
					{
						node.InnerText=sInnerText;
					}
				}
				catch ( System.Exception)
				{
					XmlNode node=doc.SelectSingleNode(sXpathStatement);

					if (node!= null)
					{
						node.InnerText=sInnerText;
					}
				}
			}
			
		}

		private void XSLT_trasform(string sSource, string sXslt, string sDestination)
		{
			//XmlTextReader tr11=new XmlTextReader(sSource);
			//XPathDocument xPathDocument=new XPathDocument(tr11);
			XslTransform xslt = new XslTransform ();
			//string nomexsl=returnValue.Value +".xsl";
			WAPI.Response wapiResponse= new WAPI.Response();

			string sDirectory = WAPI.Session.GetDirectory("WF_DMPATH", ref wapiResponse);
			if ( wapiResponse.iCode != 0 )
			{
				sDirectory =WAPI.Session.GetScenarioDBDirectory(ref wapiResponse);
			}

			sXslt=  sDirectory + "\\DM\\External\\"+sXslt;
			xslt.Load(sXslt);
			//create the output stream
			//StringBuilder sb=new StringBuilder();
			//TextWriter tw=new StringWriter(sb);
			//xslt.Transform(xPathDocument,null, tw,null);
			xslt.Transform(sSource,sDestination,null);
			
		}
		/*
		private void SelectActionInput(XMLDocumentManagementAction dmAction,string sFileName,XmlDocument doc)
		{
			switch (dmAction.Type)
			{
				case XMLDocumentManagementAction.XPATH_VALUE:
					doc.Load(sFileName);
					XmlNode node= doc.SelectSingleNode(dmAction.sStatement,null);
					if (node!= null)
					{
						node.InnerText=this.GetAttribute(dmAction.ProcessAtribute,FormHelper.ACTION_PROCESSATTRIBUTES);
						//		exitStatus=node.InnerText;
					}
					break;
				case XMLDocumentManagementAction.XSLT_VALUE:
					XmlTextReader tr11=new XmlTextReader(sFileName);
					XPathDocument xPathDocument=new XPathDocument(tr11);
					XslTransform xslt = new XslTransform ();
					string nomexsl=dmAction.Statement +".xsl";
					xslt.Load(nomexsl);
					//create the output stream
					StringBuilder sb=new StringBuilder();
					TextWriter tw=new StringWriter(sb);
					xslt.Transform(xPathDocument,null, tw,null);			
					//exitStatus=tw.ToString();
					break;
					//else exitStatus= "KO";
				case XMLDocumentManagementAction.SQL_VALUE:
					break;
			}
		}

		private void SelectActionOutput(XMLDocumentManagementAction dmAction, string sFileName,XmlDocument doc)
		{
			switch (dmAction.Type)
			{
				case XMLDocumentManagementAction.XPATH_VALUE:
					doc.Load(sFileName);
					XmlNode node= doc.SelectSingleNode(dmAction.sStatement,null);
					if (node!= null)
					{
						this.SetAttribute(dmAction.ProcessAtribute,node.InnerText,FormHelper.ACTION_PROCESSATTRIBUTES);
						//		exitStatus=node.InnerText;
					}
					break;
				case XMLDocumentManagementAction.XSLT_VALUE:
					XmlTextReader tr11=new XmlTextReader(sFileName);
					XPathDocument xPathDocument=new XPathDocument(tr11);
					XslTransform xslt = new XslTransform ();
					string nomexsl=dmAction.Statement +".xsl";
					xslt.Load(nomexsl);
					//create the output stream
					StringBuilder sb=new StringBuilder();
					TextWriter tw=new StringWriter(sb);
					xslt.Transform(xPathDocument,null, tw,null);
					//exitStatus=tw.ToString();
					break;
					//else exitStatus= "KO";
				case XMLDocumentManagementAction.SQL_VALUE:
					break;
			}
		}
		*/
		private string resolveBracketSquare(string  sString)
		{
			//Se nella stringa sono presenti parentesi quadre
			// la stringa all'interno di esse  il nome dell'
			//attributo di processo da cui prelevare il dato
			int iSquareBegin=sString.IndexOf("[");
			if (iSquareBegin>=0)
			{
				
				//Trova il nome dell'attributo di processo
				sString= sString.Substring(iSquareBegin+1,sString.Length-2);
				return(this.GetAttribute(sString,FormHelper.ACTION_PROCESSATTRIBUTES));
			}

			return sString;
		}

		private string resolveAZSquare(string  sString)
		{
			//Se nella stringa sono presenti parentesi quadre
			// la stringa all'interno di esse  il nome dell'
			//attributo di processo da cui prelevare il dato
			int iSquareBegin=sString.IndexOf("AZ");
			if (iSquareBegin>=0)
			{
				int iPosEnd = sString.IndexOf ("AZ", iSquareBegin+1);

				string sValue= sString.Substring(iSquareBegin+2,iPosEnd - iSquareBegin - 2);
				sValue = this.GetAttribute(sValue,FormHelper.ACTION_PROCESSATTRIBUTES);
				sString = sString.Substring (0, iSquareBegin) + sValue + sString.Substring (iPosEnd+2); 
				
				return resolveAZSquare(sString);
			}

			return sString;
		}

		public XMLAction GetAction(string ActionName)
		{
			string sResult=string.Empty;
			if ( (ActionName==null) || (ActionName.CompareTo (string.Empty) == 0 ) )
			{
				throw (new ArgumentOutOfRangeException("FormHelper - No ActionName"));
			}

			XMLAction action = null;
			try
			{
				action = allowedActions.Get(ActionName);
			}
			catch(Exception.XMLActionNotFound)
			{
			}

			return action;
		}


		public string GetSpecialAttribute(string sAttributeName, string ActionName, string ProcessId, string ActivityName, string ActivityResult, string User)            

		{

			string AtributeValue = string.Empty ;

                                   

			if (FH_ActivityAttributes!=null)

			{

				try

				{

					// search attribute in Module activity atributes

					XMLAttribute attr = new XMLAttribute();

					attr = FH_ActivityAttributes.Get(sAttributeName);

                                                           

					if ( attr != null )

					{

						AtributeValue = attr.Value;

						return AtributeValue;

					}

				}

				catch (ScenarioFramework.WorkflowEngine.DataModel.Exception.AttributeNotFound)

				{

					//return string.Empty;

				}

			}

 

			if (FH_ProcessAttributes!=null)

			{

				try

				{

					// search attribute in Module process atributes

					XMLAttribute attr = new XMLAttribute();

					attr = FH_ProcessAttributes.Get(sAttributeName);

					if ( attr != null )

					{

						AtributeValue = attr.Value;

						return AtributeValue;

					}

				}

				catch (ScenarioFramework.WorkflowEngine.DataModel.Exception.AttributeNotFound)

				{

					//return string.Empty;

				}

			}

                                   

			switch ( sAttributeName )

			{

				case "WFMS_USER":

					return User;

                                                           

				case "WFMS_ACTIONNAME":

					return ActionName;

                                                           

				case "WFMS_ACTIVITYNAME":

					return ActivityName;

			}

			return string.Empty;

 

		}

		public string OnAction(string ActionName, string ProcessId, string ActivityName, string ActivityResult, string User, ref string sRedirectUrl, bool bActionPriority)
		{
			string sResult=string.Empty;
			bool bAttributeToSave=false;
			if ( (ActionName==null) || (ActionName.CompareTo (string.Empty) == 0 ) )
			{
				throw (new ArgumentOutOfRangeException("FormHelper - No ActionName"));
			}

			// find action from allowed colletion
			XMLAction action = null;
			try
			{
				action = allowedActions.Get(ActionName);
			}
			catch(Exception.XMLActionNotFound)
			{
				#region try to load "form.aspx" stored in "form" attribute
				if ( ProcessId != "" && ActivityName != "")
				{
					XMLActivity activity = new XMLActivity();
					activity.ProcessId = System.Int32.Parse (ProcessId);
					activity.Name = ActivityName;

					WAPI.Response wapiResponse = new WAPI.Response();
					string sAttributes = WAPI.WorkItem.GetWorkItemAttributes (activity.GetXML(), User, ref wapiResponse);

					if ( wapiResponse.iCode == 0 )
					{
						activity.XMLAttributes.Parse (sAttributes);
						XMLAttribute formAttribute = (XMLAttribute)activity.XMLAttributes.Get("Form");
						if ( formAttribute != null )
						{
							return formAttribute.Value;
						}
					}
				}
				return "NO ACTION ALLOWED";
				#endregion
			}

			if ( bActionPriority )
			{
				this.ExecuteAction (User, ActionName, ProcessId, ActivityName);
			}

			bool bNotErr=true;
			
			FormAttributeAction formAttributeAction= new FormAttributeAction();
			if ( (action.formAttributes.Count>0) && (action.Resouces.Count>0) )
			{
				DocumentManagerDBHelper dmDBHelper = new DocumentManagerDBHelper();
				DocumentHelper docHelper=new DocumentHelper (dmDBHelper);
				string doc2load=string.Empty;
			
				#region Add IN/OUT FormAttribute
				for(int i=0; i<action.formAttributes.Count; i++)
				{
					XMLFormAttribute fa=(XMLFormAttribute)action.formAttributes[i];
					string sName = this.resolveBracketSquare(fa.Name);

					switch (fa.sDirection)
					{
						case XMLFormAttribute.INPUT_DIRECTION_VALUE:
							formAttributeAction.AddIN(new FormAttributeOperation(sName,"",fa.sStatement,fa.Resource));
							break;
						case XMLFormAttribute.OUTPUT_DIRECTION_VALUE:
							formAttributeAction.AddOUT(new FormAttributeOperation(sName,fa.sStatement,"",fa.Resource));
							break;
					}
				}
				#endregion

				#region Setting From/To field for AttributeOperation from Resource
				int idFao=-1;
				for(int i=0; i<action.Resouces.Count; i++)
				{
					XMLResource res=(XMLResource)action.Resouces[i];
					string sName = this.resolveBracketSquare(res.Id);

					idFao=formAttributeAction.SearchDocIDForIN(sName,res.NameNode);
					if (idFao>=0)
					{
						formAttributeAction.In.setFrom(idFao, res.NodeValue);
						if (res.NameNode==XMLResource.DOCUMENT_NODE)
						{
							sName = this.resolveBracketSquare(res.NameDoc);
							formAttributeAction.In.setDocName(idFao,sName);
						}
					}
					else
					{
						idFao=formAttributeAction.SearchDocIDForOUT(sName,res.NameNode);
						if (idFao>=0)
						{
							formAttributeAction.Out.setTo(idFao, res.NodeValue);
							if (res.NameNode==XMLResource.DOCUMENT_NODE)
							{
								formAttributeAction.Out.setDocName(idFao,res.NameDoc);
							}
						}
						else 
						{ 
							return "KO";
						}
					}
				
				}
				#endregion

				FormAttributeOperations formAttrOps = formAttributeAction.In;
				
				#region Only In
				if (formAttrOps.Count>0)
				{
					formAttrOps.SortByResource();
					string docLoaded=string.Empty;
					
					XmlDocument doc=new XmlDocument();

					#region Setting All "From" inside operation
					foreach (FormAttributeOperation o in formAttrOps)
					{
						switch ( o.Resource )
						{
							case XMLResource.DOCUMENT_NODE:
								if (docLoaded != o.DocName)
								{
									doc2load=docHelper.GetInstanceFileName(o.DocName);		
									doc.Load(doc2load);
									docLoaded=o.DocName;
								}
								o.From=this.readTag(o.From,doc);					
								break;

							case XMLResource.PROCESSATTRIBUTE_NODE:
								o.From=this.GetSpecialAttribute(o.From,ActionName,ProcessId,ActivityName,ActivityResult,User);				
								break;

							case XMLResource.SQL_NODE:
								string sAssembly= o.From.Substring(0,o.From.IndexOf("."))+".dll";
								string sClassName=o.From.Substring(o.From.IndexOf(".")+1,o.From.LastIndexOf(".")-o.From.IndexOf(".")-1);
								string sMethodName= o.From.Substring(o.From.LastIndexOf(".")+1);
								//gestione argomenti (vengono cercati negli attributi di processo
								int beginSquare=sMethodName.IndexOf("[");
								object [] arguments;
								if (beginSquare>=0)
								{
									string substring= sMethodName.Substring(beginSquare);
									sMethodName= sMethodName.Substring(0,sMethodName.IndexOf("["));
									System.Collections.ArrayList arg= new System.Collections.ArrayList();
									while (substring.Length>0)
									{
										string sub=substring.Substring(1,substring.IndexOf("]")-1);
										arg.Add(sub);
										substring= substring.Substring(substring.IndexOf("]")+1,(substring.Length-sub.Length-2));
									}
									arguments = new object[arg.Count];
									for(int i=0; i<arg.Count;i++)
									{
										arguments[i]=this.GetAttribute((string)arg[i],FormHelper.ACTION_PROCESSATTRIBUTES);
									}
								}
								else {arguments=null;}

								o.From= (string) bOSdotNET.CLR.AssemblyManager.InvokeMethod(sAssembly,sClassName,sMethodName,arguments);
								break;
							
							case XMLResource.XSLT_NODE:
								string sSource=docHelper.GetInstanceFileName(o.From);
								string sDestination=docHelper.GetInstanceFileName(o.To);
								this.XSLT_trasform(sSource,o.DocID,sDestination);
								break;
						}
						
					}
					#endregion

					#region Save all Document 
					formAttrOps.SortByDocID();
					docLoaded=string.Empty;
					doc=new XmlDocument();					
					foreach (FormAttributeOperation o in formAttrOps)
					{
						if (o.Resource!= XMLResource.XSLT_NODE)
						{
							if (docLoaded!= o.DocID)
							{
								if (docLoaded != string.Empty)
								{
									doc.Save(doc2load);
								}
							
								doc2load=docHelper.GetInstanceFileName(o.DocID);		
								doc.Load(doc2load);			
								docLoaded=o.DocID;
							}
							this.writeTag(o.To,doc,o.From);
						}
	
						doc.Save(doc2load);

					}
					#endregion

				}
				#endregion


				formAttrOps= formAttributeAction.Out;
				if (formAttrOps.Count>0)
				{
					formAttrOps.SortByDocID();
					string docLoaded=string.Empty;
					XmlDocument doc=new XmlDocument();					
					foreach (FormAttributeOperation o in formAttrOps)
					{
						if(!o.DocID.EndsWith("xsl"))
						{
							if (docLoaded!= o.DocID)
							{
								string sFileName = docHelper.GetInstanceFileNameWithoutPath(o.DocID);
								doc2load=docHelper.GetInstanceFileName(o.DocID);		
								doc.Load(doc2load);						
								docLoaded=o.DocID;
							}
							o.From=this.readTag(o.From,doc);
						}
						else
						{
							o.From=this.resolveBracketSquare(o.From);
						}
					}				
				
					formAttrOps.SortByResource();
					
					//Output degli attributi del documento
					int FAcount=0;
					int FAfound=0;
					string sAssembly= string.Empty;
					string sClassName=string.Empty;
					string sMethodName= string.Empty;
					System.Collections.ArrayList arg= new System.Collections.ArrayList();
					foreach (FormAttributeOperation o in formAttrOps)
					{
						docLoaded=string.Empty;

						switch ( o.Resource )
						{
							case XMLResource.DOCUMENT_NODE:
								if (docLoaded != o.DocName)
								{
									if (docLoaded != string.Empty)
									{
										doc.Save(doc2load);
									}
									string sFileName = docHelper.GetInstanceFileNameWithoutPath(o.DocName);
									doc2load=docHelper.GetInstanceFileName(o.DocName);		
									doc.Load(doc2load);	
									docLoaded=o.DocName;
								}
								this.writeTag(o.To,doc,o.From);
								break;
							
							case XMLResource.PROCESSATTRIBUTE_NODE:
								this.SetAttribute(o.To,o.From,FormHelper.ACTION_PROCESSATTRIBUTES);
								bAttributeToSave=true;
								break;
							
							case XMLResource.SQL_NODE:
								object [] arguments;
								if (FAcount ==0)
								{
									sAssembly= o.To.Substring(0,o.To.IndexOf("."))+".dll";
									sClassName=o.To.Substring(o.To.IndexOf(".")+1,o.To.LastIndexOf(".")-o.To.IndexOf(".")-1);
									sMethodName= o.To.Substring(o.To.LastIndexOf(".")+1);
									int beginSquare=sMethodName.IndexOf("[");
									if (beginSquare>=0)
									{
										string substring= sMethodName.Substring(beginSquare);
										sMethodName= sMethodName.Substring(0,sMethodName.IndexOf("["));
										while (substring.Length>0)
										{
											string sub=substring.Substring(1,substring.IndexOf("]")-1);
											arg.Add(sub);
											substring= substring.Substring(substring.IndexOf("]")+1,(substring.Length-sub.Length-2));
										}
										arguments = new object [arg.Count];
										for(int i=0; i<arg.Count;i++)
										{
											arguments[i]=this.GetAttribute((string)arg[i],FormHelper.ACTION_PROCESSATTRIBUTES);
										}
									}
									else 
									{
										beginSquare=sMethodName.IndexOf("(");
										if (beginSquare>=0)
										{
											string substring= sMethodName.Substring(beginSquare);
											sMethodName= sMethodName.Substring(0,sMethodName.IndexOf("("));
										
											//System.Collections.ArrayList arg= new System.Collections.ArrayList();
											while (substring.Length>0)
											{
												string sub=substring.Substring(1,substring.IndexOf(")")-1);
												//arg.Add(sub);
												substring= substring.Substring(substring.IndexOf(")")+1,(substring.Length-sub.Length-2));
												FAfound++;
											}
											arg.Add(o.From);
											FAcount++;
											//arguments= arg.ToArray();
										}
										else
										{
				
											arguments=null;
										}
									}
								}
								else if (FAfound!=FAcount)
								{
									arg.Add(o.From);
									FAcount++;
								}
								if(FAfound==FAcount)
								{
									arguments= arg.ToArray();
									try
									{
										WAPI.Response wapiResponse = new WAPI.Response();
										string sScenarioBin = WAPI.Session.GetScenarioExecutionDirectory(ref wapiResponse);
										if ( wapiResponse.iCode == 0 )
											bOSdotNET.CLR.AssemblyManager.InvokeMethod(sScenarioBin + "\\bin\\" + sAssembly,sClassName,sMethodName,arguments);
										else
											bOSdotNET.CLR.AssemblyManager.InvokeMethod(sAssembly,sClassName,sMethodName,arguments);
									}
									catch ( System.Exception err )
									{
										string sErr = err.Message;
									}

									FAfound=0;
									FAcount=0;
								}
								break;

							case XMLResource.XSLT_NODE:
								string sSource=docHelper.GetInstanceFileName(o.From);
								string sDestination=docHelper.GetInstanceFileName(this.resolveBracketSquare(o.To));
								this.XSLT_trasform(sSource,o.DocID,sDestination);
								break;
						}
					}

					if (docLoaded != string.Empty)
					{
						doc.Save(doc2load);
					}
					
				}//End OUT

				//save Process and Activity Attributes
				if(bAttributeToSave)
				{
					try
					{
						this.SaveXmlAttributes (ProcessId, ActivityName, User, ACTION_PROCESSATTRIBUTES);
					}
					catch
					{}
					try
					{
						this.SaveXmlAttributes (ProcessId, ActivityName, User, ACTION_ACTIVITYATTRIBUTES);
					}
					catch
					{}
				}

			}

			if ( !bActionPriority )
			{
				if ( bNotErr )
				{
					this.ExecuteAction (User, ActionName, ProcessId, ActivityName);
				}
				else
				{
					this.ExecuteActionKO (User, ActionName, ProcessId, ActivityName);
					sResult = "KO";
				}
			}

			if (!((action.RedirectUrl == null) || (action.RedirectUrl.Length ==0)))
			{
				sRedirectUrl = this.GetRedirectParameter(action.RedirectUrl);
			}

			return sResult;
		}
	}
}
