using System;

namespace ScenarioFramework.WorkflowEngine.DataModel.Common
{
	//*********************************************************************
	//
	// FormalParameter Class
	//
	// toto : inserire l'esatta descrizione di ci che fa l'oggetto
	//
	//*********************************************************************

	public class XMLFormalParameter : bOSdotNET.XML.Data.XMLObject 
	{
		public const string MAIN_NODE			= "Parameter";
		
		private const string NAME_ATTRIBUTE		= "Name";
		private const string TYPE_ATTRIBUTE		= "Type";
		private const string MANDATORY_ATTRIBUTE= "Mandatory";
		private const string VALUE_NODE			= "Value";
		private const string BIND_NODE			= "Bind";


		private const string MANDATORY_YES		= "YES";
		private const string MANDATORY_NO		= "NO";

		public const string XML_TYPE = "XML";
		public const string STRING_TYPE = "STRING";

		/** FIELDS **/

		private string sName;
		private string sType;
		private string sValue;
		private bool bMandatory;
		private string sBind;

		/** CONSTRUCTORS **/

		/** CONSTRUCTOR Width simple Property **/
		public XMLFormalParameter()
		{
			this.Init();
		}

		/** CONSTRUCTOR Width xml parsering**/
		public XMLFormalParameter(System.Xml.XmlNode node)
		{
			this.Parse(node);
		}

		/** CONSTRUCTOR Width xml Values **/
		public XMLFormalParameter(string _Name, string _Value, string _Type)
		{
			this.Name = _Name;
			this.Value = _Value;
			this.Type = _Type;
		}

		/** PROPERTIES **/


		public string Name 
		{
			get { return sName; }
			set { sName = value; }

		}

		public string Type 
		{
			get { return sType; }
			set { sType = value; }

		}

		public string Value 
		{
			get { return sValue; }
			set { sValue = value; }

		}

		public string Bind 
		{
			get { return sBind; }
			set { sBind = value; }

		}

		public bool Mandatory 
		{
			get { return bMandatory; }
			set { bMandatory = value; }

		}

		/** INSTANCE METHODS **/

		
		override public void Init()
		{
			sName= string.Empty;
			sValue= string.Empty;
			sType= string.Empty;
			sBind = string.Empty;
			bMandatory=false;
		}

		/** Parsering method to fill the object **/
		override public void Parse(System.Xml.XmlNode node2Parse)
		{
			System.Xml.XmlNode node;

			/** find the attributes situated on the root tag **/
			for ( int i=0; i<node2Parse.Attributes.Count;i++)
			{
				node = node2Parse.Attributes.Item(i);
				switch ( node.Name ) 
				{
					case XMLFormalParameter.NAME_ATTRIBUTE:
						sName = node.InnerText;
						break;

					case XMLFormalParameter.MANDATORY_ATTRIBUTE:
						if ( node.InnerText == MANDATORY_YES )
						{
							bMandatory = true;
						}
						break;
					case ScenarioFramework.WorkflowEngine.DataModel.Common.XMLFormalParameter.TYPE_ATTRIBUTE:
						sType = node.InnerText;
						break;
				}
			}

			/** find the attributes situated on sub tags **/
			for ( int i=0; i<node2Parse.ChildNodes.Count;i++)
			{
				node = node2Parse.ChildNodes[i];

				switch ( node.Name) 
				{
					case XMLFormalParameter.BIND_NODE:
						this.sBind  = node.InnerText;
						break;
					case XMLFormalParameter.VALUE_NODE:
						if ( sType == XMLFormalParameter.XML_TYPE )
						{
							sValue = node.InnerXml;
						}
						else
						{
							sValue = node.InnerText;
						}
						break;
				}
			}
		}

		/**Method to retive a node from object **/
		override public bOSdotNET.XML.XMLNode GetXMLNode()
		{
			bOSdotNET.XML.XMLNode xmlNode = new bOSdotNET.XML.XMLNode(XMLFormalParameter.MAIN_NODE);
			xmlNode.AddAttribute ( new bOSdotNET.XML.XMLAttribute ( XMLFormalParameter.NAME_ATTRIBUTE, sName) );
			xmlNode.AddAttribute ( new bOSdotNET.XML.XMLAttribute ( XMLFormalParameter.MANDATORY_ATTRIBUTE, bMandatory ? MANDATORY_YES : MANDATORY_NO) );
			xmlNode.AddAttribute ( new bOSdotNET.XML.XMLAttribute ( XMLFormalParameter.TYPE_ATTRIBUTE, sType) );

			xmlNode.AddNode ( new bOSdotNET.XML.XMLNode ( XMLFormalParameter.BIND_NODE, sBind) );
			xmlNode.AddNode ( new bOSdotNET.XML.XMLNode ( XMLFormalParameter.VALUE_NODE, sValue) );

			return xmlNode;
		}
	}

}