using System;
using ScenarioFramework.WorkflowEngine.DataModel;

namespace ScenarioFramework.WorkflowEngine.DataModel.WorkflowProcesses
{
	//*********************************************************************
	//
	// Activity Class
	//
	// The XmlActivity class represents a single task of a workflow template
	//
	//*********************************************************************

	public class XMLActivity : bOSdotNET.XML.Data.XMLObject 
	{
		public const string MAIN_NODE					= "Activity";

		private const string NAME_ATTRIBUTE				= "Name";
		private const string TYPE_ATTRIBUTE				= "Type";

		private const string DESCRIPTION_NODE			= "Description";
		private const string REFERENCE_ACTIVITY_NODE	= "ReferenceActivity";
		private const string CATCH_TYPE_NODE			= "CatchType";
		
		public static string START_NAME					= "Start";
		public static string END_NAME					= "End";

		public const string START_TYPE			= "START";
		public const string END_TYPE			= "END";
		public const string SUBFLOW_TYPE		= "SUBFLOW";
		public const string API_TYPE			= "API";
		public const string CONDITION_TYPE		= "CONDITION";
		public const string EMAIL_TYPE			= "EMAIL";
		public const string APPLICATION_TYPE	= "APPLICATION";
		public const string XML_TYPE			= "XML";
		public const string EXIT_TYPE			= "EXIT";
		public const string EVENT_TYPE			= "EVENT";
		public const string THROWER_TYPE		= "THROWER";
		public const string CATCHER_TYPE		= "CATCHER";

		public const string NEVER_CATCH_TYPE		= "NEVER";
		public const string EVER_CATCH_TYPE			= "EVER";
		public const string ONLY_ONE_CATCH_TYPE		= "ONLYONE";

		//TODO: XMLProcess to develop.
		private const String PROCESS_ID_NODE		= "ProcessId";

		//Attributi che specializzano attivit

		public const string MARKER_FEATURE			= "WFMS_FEATURE";
		public const string MARKER_NOTIFY_FEATURE	= "NOTIFY";
		public const string MARKER_ASSIGN_TO_MARKER	= "WFMS_ASSIGN_TO";

		//SubFLow
		public const string MARKER_OCCURENCY = "Occurency";
		public const string MARKER_EXECUTION = "Execution";
		public const string MARKER_PROCESSNAME = "ProcessName";
		public const string MARKER_SEQUENCE_EXECUTION = "SEQUENCE";

		private string sName;
		private string sType;
		private string sDescription;
		private string sReferenceActivity;
		private string sCatchType;
		private long lProcessId;

		private XMLBehaviour behaviour;
		private ScenarioFramework.WorkflowEngine.DataModel.Common.XMLGrants grants;
		private WorkflowRelevantData.XMLAttributes vVariables;
		private XMLLog xmlLog;

		/** CONSTRUCTORS **/

		/** CONSTRUCTOR Width simple Property **/
		public XMLActivity()
		{
			this.Init();
		}

		/** CONSTRUCTOR Width xml parsering**/
		public XMLActivity(System.Xml.XmlNode node)
		{
			this.Parse(node);
		}

		public XMLActivity(string _Name)
		{
			sName=_Name;
		}

		/** PROPERTIES **/
		public XMLBehaviour XMLBehaviour
		{
			get { return behaviour;}
			set { behaviour = value;}
		}

		public Common.XMLGrants XMLGrants
		{
			get { return grants; }
			set { grants = value; }
		}

		public WorkflowRelevantData.XMLAttributes XMLAttributes
		{
			get { return vVariables; }
			set { vVariables = value; }
		}

		public XMLLog XMLLog
		{
			get {return xmlLog;}
			set {xmlLog = value;}
		}

		public long ProcessId
		{
			get { return lProcessId;}
			set { lProcessId = value;}
		}

		public string Name 
		{
			get { return sName; }
			set { sName = value; }
		}

		public string Type 
		{
			get { return sType; }
			set { sType = value; }
		}

		public string Description 
		{
			get { return sDescription; }
			set { sDescription = value; }
		}

		public string ReferenceActivity 
		{
			get { return sReferenceActivity; }
			set { sReferenceActivity = value; }
		}

		public string CatchType
		{
			get { return sCatchType; }
			set { sCatchType = value; }
		}
		
		public string Date
		{
			get
			{
				return xmlLog.Date;
			}
		}

		/** INSTANCE METHODS **/

		override public void Init()
		{
			sName = string.Empty;

			sType = string.Empty;
			sDescription = string.Empty;
			sReferenceActivity = string.Empty;
			sCatchType = string.Empty;
			lProcessId = 0;

			behaviour = new XMLBehaviour();
			grants = new ScenarioFramework.WorkflowEngine.DataModel.Common.XMLGrants();
			vVariables = new WorkflowRelevantData.XMLAttributes();

			xmlLog = new XMLLog();
		}

		override public void Parse(System.Xml.XmlNode node2Parse)
		{
			this.Init();

			System.Xml.XmlNode node;

			
			/** find the attributes situated on the root tag **/
			for ( int i=0; i<node2Parse.Attributes.Count;i++)
			{
				node = node2Parse.Attributes.Item(i);

				switch ( node.Name )
				{
					case XMLActivity.NAME_ATTRIBUTE:
						sName = node.InnerText;
						break;

					case XMLActivity.TYPE_ATTRIBUTE:
						sType = node.InnerText;
						break;
				}
			}

			/** find the sub tags witch contains values **/
			for ( int i=0; i<node2Parse.ChildNodes.Count; i++)
			{
				node = node2Parse.ChildNodes[i];

				switch ( node.Name )
				{
					case XMLActivity.DESCRIPTION_NODE:
						sDescription= node.InnerText;
						break;

					case XMLActivity.CATCH_TYPE_NODE:
						sCatchType = node.InnerText;
						break;

					case XMLActivity.REFERENCE_ACTIVITY_NODE:
						sReferenceActivity=node.InnerText;
						break;

					case ScenarioFramework.WorkflowEngine.DataModel.Common.XMLGrants.MAIN_NODE:
						grants.Parse(node);
						break;
					
					case ScenarioFramework.WorkflowEngine.DataModel.WorkflowRelevantData.XMLAttributes.MAIN_NODE:
						vVariables.Parse(node);
						break;

					case XMLBehaviour.MAIN_NODE:
						behaviour.Parse(node);
						break;

					case XMLActivity.PROCESS_ID_NODE:
						lProcessId = Int32.Parse ( node.InnerText);
						break;

					case XMLLog.MAIN_NODE:
						xmlLog.Parse (node);
						break;

				}
			}
		}

		override public bOSdotNET.XML.XMLNode GetXMLNode()
		{
			bOSdotNET.XML.XMLNode xmlNode = new bOSdotNET.XML.XMLNode(XMLActivity.MAIN_NODE);

			xmlNode.AddAttribute ( new bOSdotNET.XML.XMLAttribute (XMLActivity.NAME_ATTRIBUTE, sName) );
			xmlNode.AddAttribute ( new bOSdotNET.XML.XMLAttribute (XMLActivity.TYPE_ATTRIBUTE, sType) );
			xmlNode.AddNode ( new bOSdotNET.XML.XMLNode (XMLActivity.DESCRIPTION_NODE, sDescription) );
			xmlNode.AddNode ( new bOSdotNET.XML.XMLNode (XMLActivity.PROCESS_ID_NODE, lProcessId) );
			xmlNode.AddNode ( new bOSdotNET.XML.XMLNode (XMLActivity.CATCH_TYPE_NODE, sCatchType) );
			xmlNode.AddNode ( new bOSdotNET.XML.XMLNode (XMLActivity.REFERENCE_ACTIVITY_NODE, sReferenceActivity) );
			
			xmlNode.AddNode (grants.GetXMLNode() );
			xmlNode.AddNode (vVariables.GetXMLNode() );
			xmlNode.AddNode (behaviour.GetXMLNode());
			xmlNode.AddNode (xmlLog.GetXMLNode());
			
			return xmlNode;
		}

		
		public void SetToStart()
		{
			this.sName=XMLActivity.START_NAME;
			this.sType=XMLActivity.START_TYPE;
			this.sDescription="Start Task";

			this.behaviour.XMLBehaviourStart.Mode = "AUTOMATIC";
			this.behaviour.XMLBehaviourStart.OnlyOne = "YES";
			this.behaviour.XMLBehaviourStart.Report = "Activity START for Process [WorkflowProcess.Name] STARTED ";

			this.behaviour.XMLBehaviourFinish.Mode = "AUTOMATIC";
			this.behaviour.XMLBehaviourFinish.Report = "Activity START for Process [WorkflowProcess.Name] ENDED ";

			this.behaviour.XMLBehaviourExecution.Mode = "AUTOMATIC";
			this.behaviour.XMLBehaviourExecution.Type = "IMPLEMENTATION";
			this.behaviour.XMLBehaviourExecution.Report = "Activity START for the Process [WorkflowProcess.Name] IN EXECUTION";
			this.behaviour.XMLBehaviourExecution.XMLScript.Set2None();
			
		}

		public void SetToEnd()
		{
			this.sName=XMLActivity.END_NAME;
			this.sType=XMLActivity.END_TYPE;
			this.sDescription="End Task";

			this.behaviour.XMLBehaviourStart.Mode = XMLBehaviourStart.AUTOMATIC_MODE;
			this.behaviour.XMLBehaviourStart.OnlyOne = XMLBehaviourStart.YES_ONLYONE;
			this.behaviour.XMLBehaviourStart.Report = "Activity END for Process [WorkflowProcess.Name] STARTED ";

			this.behaviour.XMLBehaviourFinish.Mode = XMLBehaviourFinish.AUTOMATIC_MODE;
			this.behaviour.XMLBehaviourFinish.Report = "Activity END for Process [WorkflowProcess.Name] ENDED ";

			this.behaviour.XMLBehaviourExecution.Mode = XMLBehaviourExecution.AUTOMATIC_MODE;
			this.behaviour.XMLBehaviourExecution.Type = XMLBehaviourExecution.IMPLEMENTATION_TYPE;
			this.behaviour.XMLBehaviourExecution.Report = "Activity END for the Process [WorkflowProcess.Name] IN EXECUTION";
			this.behaviour.XMLBehaviourExecution.XMLScript.Set2None();
		}

		public void SetToNoneAction()
		{
			this.behaviour.XMLBehaviourExecution.XMLScript.Set2None();
		}

		static public WorkflowRelevantData.XMLAttributes GetEmbeddedAttributes(string sActivityType)
		{
			WorkflowRelevantData.XMLAttributes vEmbeddedAttributes = new WorkflowRelevantData.XMLAttributes();

			switch ( sActivityType )
			{
				case START_TYPE:
				case END_TYPE:
					break;

				case SUBFLOW_TYPE:
					WorkflowRelevantData.XMLAttribute attribute = new WorkflowRelevantData.XMLAttribute();
					attribute.Name =WorkflowProcesses.XMLActivity.MARKER_PROCESSNAME;
					attribute.Type = WorkflowDataTypes.XMLDataType.WORKFLOWPROCESSES_DATATYPE;
					attribute.Direction = WorkflowRelevantData.XMLAttribute.DIRECTION_INPUT;
					vEmbeddedAttributes.Add ( attribute);

					attribute = new WorkflowRelevantData.XMLAttribute();
					attribute.Name =WorkflowProcesses.XMLActivity.MARKER_OCCURENCY;
					attribute.Type = WorkflowDataTypes.XMLDataType.NUMBER_DATATYPE;
					attribute.Direction = WorkflowRelevantData.XMLAttribute.DIRECTION_INPUT;
					vEmbeddedAttributes.Add ( attribute);

					attribute = new WorkflowRelevantData.XMLAttribute();
					attribute.Name =WorkflowProcesses.XMLActivity.MARKER_EXECUTION;
					attribute.Type = WorkflowDataTypes.XMLDataType.SUBFLOWEXECUTION_DATATYPE;
					attribute.Direction = WorkflowRelevantData.XMLAttribute.DIRECTION_INPUT;
					vEmbeddedAttributes.Add ( attribute);
					break;

				case API_TYPE:
				case CONDITION_TYPE:
					break;

				case EMAIL_TYPE:
					break;

				case APPLICATION_TYPE:
				case XML_TYPE:
				case EXIT_TYPE:
				case EVENT_TYPE:
				case THROWER_TYPE:
				case CATCHER_TYPE:
					break;
			}

			return vEmbeddedAttributes;
		}

	}
}