using System;
using System.Collections;
using ScenarioFramework.WorkflowEngine.DataModel;

namespace ScenarioFramework.WorkflowEngine.DataModel.WorkflowServices
{
	//*********************************************************************
	//
	// XMLServices Class
	//
	// The XMLServices class represents a collection base of XMLService
	// Its imports methods from WAPI.XmlObjectSet like get set and add
	// XMLServices Has no fields
	//
	//*********************************************************************
	public class XMLServices : bOSdotNET.XML.Data.XMLObjectSet
	{
		public const string MAIN_NODE	= "Services";

		/** FIELDS **/

		/** no fields **/

		/** CONSTRUCTORS **/

		/** CONSTRUCTOR Empty **/
		public XMLServices()
		{
			this.Init();
		}

		/** CONSTRUCTOR Width xml parsering**/
		public XMLServices(System.Xml.XmlNode nodeList)
		{
			this.Parse (nodeList);
		}

		/** INSTANCE METHODS **/
		public override void Init()
		{
			this.RemoveAll();
		}

		/**Method to parse an xml string into a nodelist collection **/
		override public void Parse(System.Xml.XmlNode nodeList)
		{
			this.Init();

			System.Xml.XmlNode node;

			for ( int i=0; i<nodeList.ChildNodes.Count;i++)
			{
				node = nodeList.ChildNodes[i];

				switch ( node.Name) 
				{
					case XMLService.MAIN_NODE:
						this.Add ( new XMLService(node));
						break;
				}
			}
		}

		/** Method for return a functionParameter from collection **/
		override public bOSdotNET.XML.XMLNode GetXMLNode()
		{
			bOSdotNET.XML.XMLNode xmlNode = new bOSdotNET.XML.XMLNode(XMLServices.MAIN_NODE);

			for ( int i=0; i<this.Count;i++)
			{
				xmlNode.AddNode ( this[i].GetXMLNode() );
			}

			return xmlNode;
		}

		/** SORTING  FIELDS **/

		public enum SortFields 
		{
			Name,
			Workflow,
			Description
		}

		/** SORTING METHODS **/


		public void Sort(SortFields sortField, bool isAscending) 
		{
			switch (sortField) 
			{
				case SortFields.Name:
					InnerList.Sort(new NameComparer());
					break;
				case SortFields.Workflow:
					InnerList.Sort(new WorkflowComparer());
					break;
				case SortFields.Description:
					InnerList.Sort(new DescriptionComparer());
					break;
				
			}
			if (!isAscending) InnerList.Reverse();
		}


		private sealed class NameComparer : IComparer 
		{
			public int Compare(object x, object y) 
			{
				XMLService first = (XMLService) x;
				XMLService second = (XMLService) y;
				return first.Name.CompareTo(second.Name);
			}
		}

		private sealed class WorkflowComparer : IComparer 
		{
			public int Compare(object x, object y) 
			{
				XMLService first = (XMLService) x;
				XMLService second = (XMLService) y;
				return first.Workflow.CompareTo(second.Workflow);
			}
		}

		private sealed class DescriptionComparer : IComparer 
		{
			public int Compare(object x, object y) 
			{
				XMLService first = (XMLService) x;
				XMLService second = (XMLService) y;
				return first.Description.CompareTo(second.Description);
			}
		}

		
	}
}
